import os


def define_maia_core_versions():

    if os.environ.get("PROMETHEUS_CHART_VERSION") is not None:
        prometheus_chart_version = os.environ.get("PROMETHEUS_CHART_VERSION")
    else:
        prometheus_chart_version = "45.5.0"

    if os.environ.get("LOKI_CHART_VERSION") is not None:
        loki_chart_version = os.environ.get("LOKI_CHART_VERSION")
    else:
        loki_chart_version = "2.9.9"

    if os.environ.get("TEMPO_CHART_VERSION") is not None:
        tempo_chart_version = os.environ.get("TEMPO_CHART_VERSION")
    else:
        tempo_chart_version = "1.0.0"

    if os.environ.get("TRAEFIK_CHART_VERSION") is not None:
        traefik_chart_version = os.environ.get("TRAEFIK_CHART_VERSION")
    else:
        traefik_chart_version = "33.2.1"

    if os.environ.get("METALLB_CHART_VERSION") is not None:
        metallb_chart_version = os.environ.get("METALLB_CHART_VERSION")
    else:
        metallb_chart_version = "0.14.9"

    if os.environ.get("CERT_MANAGER_CHART_VERSION") is not None:
        cert_manager_chart_version = os.environ.get("CERT_MANAGER_CHART_VERSION")
    else:
        cert_manager_chart_version = "1.16.2"

    if os.environ.get("GPU_OPERATOR_CHART_VERSION") is not None:
        gpu_operator_chart_version = os.environ.get("GPU_OPERATOR_CHART_VERSION")
    else:
        gpu_operator_chart_version = "25.3.1"

    if os.environ.get("INGRESS_NGINX_CHART_VERSION") is not None:
        ingress_nginx_chart_version = os.environ.get("INGRESS_NGINX_CHART_VERSION")
    else:
        ingress_nginx_chart_version = "4.11.3"

    if os.environ.get("NFS_SERVER_PROVISIONER_CHART_VERSION") is not None:
        nfs_server_provisioner_chart_version = os.environ.get("NFS_SERVER_PROVISIONER_CHART_VERSION")
    else:
        nfs_server_provisioner_chart_version = "4.0.18"

    if os.environ.get("METRICS_SERVER_CHART_VERSION") is not None:
        metrics_server_chart_version = os.environ.get("METRICS_SERVER_CHART_VERSION")
    else:
        metrics_server_chart_version = "3.13.0"

    if os.environ.get("GPU_BOOKING_CHART_VERSION") is not None:
        gpu_booking_chart_version = os.environ.get("GPU_BOOKING_CHART_VERSION")
    else:
        gpu_booking_chart_version = "master"  # "1.0.0"

    if os.environ.get("GPU_BOOKING_CHART_TYPE") is not None:
        gpu_booking_chart_type = os.environ.get("GPU_BOOKING_CHART_TYPE")
    else:
        gpu_booking_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("CORE_TOOLKIT_CHART_VERSION") is not None:
        core_toolkit_chart_version = os.environ.get("CORE_TOOLKIT_CHART_VERSION")
    else:
        core_toolkit_chart_version = "master"  # "0.2.3"

    if os.environ.get("CORE_TOOLKIT_CHART_TYPE") is not None:
        core_toolkit_chart_type = os.environ.get("CORE_TOOLKIT_CHART_TYPE")
    else:
        core_toolkit_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("LOCAL_PATH_CHART_VERSION") is not None:
        local_path_chart_version = os.environ.get("LOCAL_PATH_CHART_VERSION")
    else:
        local_path_chart_version = "master"  # "0.1.0"

    if os.environ.get("LOCAL_PATH_CHART_TYPE") is not None:
        local_path_chart_type = os.environ.get("LOCAL_PATH_CHART_TYPE")
    else:
        local_path_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("CORE_PROJECT_CHART_VERSION") is not None:
        core_project_chart_version = os.environ.get("CORE_PROJECT_CHART_VERSION")
    else:
        core_project_chart_version = "1.2.3"

    if os.environ.get("LOGINAPP_CHART_VERSION") is not None:
        loginapp_chart_version = os.environ.get("LOGINAPP_CHART_VERSION")
    else:
        loginapp_chart_version = "1.3.0"

    if os.environ.get("MINIO_OPERATOR_CHART_VERSION") is not None:
        minio_operator_chart_version = os.environ.get("MINIO_OPERATOR_CHART_VERSION")
    else:
        minio_operator_chart_version = "6.0.4"

    if os.environ.get("KUBEFLOW_CHART_VERSION") is not None:
        kubeflow_chart_version = os.environ.get("KUBEFLOW_CHART_VERSION")
    else:
        kubeflow_chart_version = "master"  # "1.0.0"

    if os.environ.get("KUBEFLOW_CHART_TYPE") is not None:
        kubeflow_chart_type = os.environ.get("KUBEFLOW_CHART_TYPE")
    else:
        kubeflow_chart_type = "git_repo"  # or "helm_repo"

    return {
        "prometheus_chart_version": prometheus_chart_version,
        "loki_chart_version": loki_chart_version,
        "tempo_chart_version": tempo_chart_version,
        "traefik_chart_version": traefik_chart_version,
        "metallb_chart_version": metallb_chart_version,
        "cert_manager_chart_version": cert_manager_chart_version,
        "gpu_operator_chart_version": gpu_operator_chart_version,
        "ingress_nginx_chart_version": ingress_nginx_chart_version,
        "nfs_server_provisioner_chart_version": nfs_server_provisioner_chart_version,
        "metrics_server_chart_version": metrics_server_chart_version,
        "gpu_booking_chart_version": gpu_booking_chart_version,
        "gpu_booking_chart_type": gpu_booking_chart_type,
        "core_project_chart_version": core_project_chart_version,
        "core_toolkit_chart_version": core_toolkit_chart_version,
        "core_toolkit_chart_type": core_toolkit_chart_type,
        "loginapp_chart_version": loginapp_chart_version,
        "minio_operator_chart_version": minio_operator_chart_version,
        "local_path_chart_version": local_path_chart_version,
        "local_path_chart_type": local_path_chart_type,
        "kubeflow_chart_version": kubeflow_chart_version,
        "kubeflow_chart_type": kubeflow_chart_type,
    }


def define_maia_admin_versions():

    if os.environ.get("RANCHER_CHART_VERSION") is not None:
        rancher_chart_version = os.environ.get("RANCHER_CHART_VERSION")
    else:
        rancher_chart_version = "2.13.0"

    if os.environ.get("HARBOR_CHART_VERSION") is not None:
        harbor_chart_version = os.environ.get("HARBOR_CHART_VERSION")
    else:
        harbor_chart_version = "1.18.2"

    if os.environ.get("KEYCLOAK_CHART_VERSION") is not None:
        keycloak_chart_version = os.environ.get("KEYCLOAK_CHART_VERSION")
    else:
        keycloak_chart_version = "24.2.0"

    if os.environ.get("ADMIN_TOOLKIT_CHART_VERSION") is not None:
        admin_toolkit_chart_version = os.environ.get("ADMIN_TOOLKIT_CHART_VERSION")
    else:
        admin_toolkit_chart_version = "master"  # "1.3.5"

    if os.environ.get("ADMIN_TOOLKIT_CHART_TYPE") is not None:
        admin_toolkit_chart_type = os.environ.get("ADMIN_TOOLKIT_CHART_TYPE")
    else:
        admin_toolkit_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("MAIA_DASHBOARD_CHART_VERSION") is not None:
        maia_dashboard_chart_version = os.environ.get("MAIA_DASHBOARD_CHART_VERSION")
    else:
        maia_dashboard_chart_version = "master"  # "0.2.2"

    if os.environ.get("MAIA_DASHBOARD_IMAGE_VERSION") is not None:
        maia_dashboard_image_version = os.environ.get("MAIA_DASHBOARD_IMAGE_VERSION")
    else:
        maia_dashboard_image_version = define_docker_image_versions()["maia-dashboard"]

    if os.environ.get("MAIA_DASHBOARD_DEV_TAG_SUFFIX") is not None:
        maia_dashboard_dev_tag_suffix = os.environ.get("MAIA_DASHBOARD_DEV_TAG_SUFFIX")
    else:
        maia_dashboard_dev_tag_suffix = "-dev"

    if os.environ.get("MAIA_DASHBOARD_CHART_TYPE") is not None:
        maia_dashboard_chart_type = os.environ.get("MAIA_DASHBOARD_CHART_TYPE")
    else:
        maia_dashboard_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("ADMIN_PROJECT_CHART_VERSION") is not None:
        admin_project_chart_version = os.environ.get("ADMIN_PROJECT_CHART_VERSION")
    else:
        admin_project_chart_version = "1.1.0"

    return {
        "rancher_chart_version": rancher_chart_version,
        "harbor_chart_version": harbor_chart_version,
        "keycloak_chart_version": keycloak_chart_version,
        "admin_toolkit_chart_version": admin_toolkit_chart_version,
        "admin_toolkit_chart_type": admin_toolkit_chart_type,
        "maia_dashboard_chart_version": maia_dashboard_chart_version,
        "maia_dashboard_image_version": maia_dashboard_image_version,
        "maia_dashboard_dev_tag_suffix": maia_dashboard_dev_tag_suffix,
        "maia_dashboard_chart_type": maia_dashboard_chart_type,
        "admin_project_chart_version": admin_project_chart_version,
    }


def define_maia_project_versions():

    if os.environ.get("MAIA_NAMESPACE_CHART_VERSION") is not None:
        maia_namespace_chart_version = os.environ.get("MAIA_NAMESPACE_CHART_VERSION")
    else:
        maia_namespace_chart_version = "master"  # "1.7.3"

    if os.environ.get("MAIA_FILEBROWSER_CHART_VERSION") is not None:
        maia_filebrowser_chart_version = os.environ.get("MAIA_FILEBROWSER_CHART_VERSION")
    else:
        maia_filebrowser_chart_version = "master"  # "1.0.0"

    if os.environ.get("MAIA_FILEBROWSER_CHART_TYPE") is not None:
        maia_filebrowser_chart_type = os.environ.get("MAIA_FILEBROWSER_CHART_TYPE")
    else:
        maia_filebrowser_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("MAIA_PROJECT_CHART_VERSION") is not None:
        maia_project_chart_version = os.environ.get("MAIA_PROJECT_CHART_VERSION")
    else:
        maia_project_chart_version = "1.9.2"

    if os.environ.get("MAIA_NAMESPACE_CHART_TYPE") is not None:
        maia_namespace_chart_type = os.environ.get("MAIA_NAMESPACE_CHART_TYPE")
    else:
        maia_namespace_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("MAIA_ORTHANC_CHART_VERSION") is not None:
        maia_orthanc_chart_version = os.environ.get("MAIA_ORTHANC_CHART_VERSION")
    else:
        maia_orthanc_chart_version = "master"  # "1.1.0"

    if os.environ.get("MAIA_ORTHANC_CHART_TYPE") is not None:
        maia_orthanc_chart_type = os.environ.get("MAIA_ORTHANC_CHART_TYPE")
    else:
        maia_orthanc_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("MAIA_KUBEFLOW_CHART_VERSION") is not None:
        maia_kubeflow_chart_version = os.environ.get("MAIA_KUBEFLOW_CHART_VERSION")
    else:
        maia_kubeflow_chart_version = "master"  # "1.0.0"

    if os.environ.get("MAIA_KUBEFLOW_CHART_TYPE") is not None:
        maia_kubeflow_chart_type = os.environ.get("MAIA_KUBEFLOW_CHART_TYPE")
    else:
        maia_kubeflow_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("MAIA_NVFLARE_DASHBOARD_CHART_VERSION") is not None:
        maia_nvflare_dashboard_chart_version = os.environ.get("MAIA_NVFLARE_DASHBOARD_CHART_VERSION")
    else:
        maia_nvflare_dashboard_chart_version = "master"  # "1.0.0"

    if os.environ.get("MAIA_NVFLARE_DASHBOARD_CHART_TYPE") is not None:
        maia_nvflare_dashboard_chart_type = os.environ.get("MAIA_NVFLARE_DASHBOARD_CHART_TYPE")
    else:
        maia_nvflare_dashboard_chart_type = "git_repo"  # or "helm_repo"

    return {
        "maia_namespace_chart_version": maia_namespace_chart_version,
        "maia_filebrowser_chart_version": maia_filebrowser_chart_version,
        "maia_filebrowser_chart_type": maia_filebrowser_chart_type,
        "maia_project_chart_version": maia_project_chart_version,
        "maia_namespace_chart_type": maia_namespace_chart_type,
        "maia-orthanc-chart_version": maia_orthanc_chart_version,
        "maia-orthanc-chart_type": maia_orthanc_chart_type,
        "maia-kubeflow-chart_version": maia_kubeflow_chart_version,
        "maia-kubeflow-chart_type": maia_kubeflow_chart_type,
        "maia-nvflare-dashboard-chart_version": maia_nvflare_dashboard_chart_version,
        "maia-nvflare-dashboard-chart_type": maia_nvflare_dashboard_chart_type,
    }


def define_maia_docker_versions():

    if os.environ.get("KANIKO_CHART_VERSION") is not None:
        kaniko_chart_version = os.environ.get("KANIKO_CHART_VERSION")
    else:
        kaniko_chart_version = "master"  # "1.0.4"

    if os.environ.get("KANIKO_CHART_TYPE") is not None:
        kaniko_chart_type = os.environ.get("KANIKO_CHART_TYPE")
    else:
        kaniko_chart_type = "git_repo"  # or "helm_repo"

    if os.environ.get("MKG_CHART_VERSION") is not None:
        mkg_chart_version = os.environ.get("MKG_CHART_VERSION")
    else:
        mkg_chart_version = "master"

    if os.environ.get("MKG_CHART_TYPE") is not None:
        mkg_chart_type = os.environ.get("MKG_CHART_TYPE")
    else:
        mkg_chart_type = "git_repo"

    return {
        "kaniko_chart_version": kaniko_chart_version,
        "kaniko_chart_type": kaniko_chart_type,
        "mkg_chart_version": mkg_chart_version,
        "mkg_chart_type": mkg_chart_type,
    }


def define_docker_image_versions():

    if os.environ.get("MAIA_KUBE_IMAGE_VERSION") is not None:
        maia_kube_image_version = os.environ.get("MAIA_KUBE_IMAGE_VERSION")
    else:
        maia_kube_image_version = "1.0"

    if os.environ.get("MAIA_DASHBOARD_IMAGE_VERSION") is not None:
        maia_dashboard_image_version = os.environ.get("MAIA_DASHBOARD_IMAGE_VERSION")
    else:
        maia_dashboard_image_version = "2.4.1"

    if os.environ.get("MONAI_TOOLKIT_IMAGE_VERSION") is not None:
        monai_toolkit_image_version = os.environ.get("MONAI_TOOLKIT_IMAGE_VERSION")
    else:
        monai_toolkit_image_version = "3.0"

    if os.environ.get("MAIA_XNAT_IMAGE_VERSION") is not None:
        maia_xnat_image_version = os.environ.get("MAIA_XNAT_IMAGE_VERSION")
    else:
        maia_xnat_image_version = "1.0"

    if os.environ.get("MAIA_ORTHANC_IMAGE_VERSION") is not None:
        maia_orthanc_image_version = os.environ.get("MAIA_ORTHANC_IMAGE_VERSION")
    else:
        maia_orthanc_image_version = "1.3"

    if os.environ.get("MAIA_MLFLOW_IMAGE_VERSION") is not None:
        maia_mlflow_image_version = os.environ.get("MAIA_MLFLOW_IMAGE_VERSION")
    else:
        maia_mlflow_image_version = "1.2"

    if os.environ.get("MAIA_FILEBROWSER_IMAGE_VERSION") is not None:
        maia_filebrowser_image_version = os.environ.get("MAIA_FILEBROWSER_IMAGE_VERSION")
    else:
        maia_filebrowser_image_version = "1.1"

    if os.environ.get("MAIA_GPU_BOOKING_ADMISSION_CONTROLLER_IMAGE_VERSION") is not None:
        maia_gpu_booking_admission_controller_image_version = os.environ.get(
            "MAIA_GPU_BOOKING_ADMISSION_CONTROLLER_IMAGE_VERSION"
        )
    else:
        maia_gpu_booking_admission_controller_image_version = "1.0"

    if os.environ.get("MAIA_GPU_BOOKING_POD_TERMINATOR_IMAGE_VERSION") is not None:
        maia_gpu_booking_pod_terminator_image_version = os.environ.get("MAIA_GPU_BOOKING_POD_TERMINATOR_IMAGE_VERSION")
    else:
        maia_gpu_booking_pod_terminator_image_version = "1.0"

    if os.environ.get("MAIA_WORKSPACE_BASE_IMAGE_VERSION") is not None:
        maia_workspace_base_image_version = os.environ.get("MAIA_WORKSPACE_BASE_IMAGE_VERSION")
    else:
        maia_workspace_base_image_version = "1.8.1"

    if os.environ.get("MAIA_WORKSPACE_BASE_NOTEBOOK_IMAGE_VERSION") is not None:
        maia_workspace_base_notebook_image_version = os.environ.get("MAIA_WORKSPACE_BASE_NOTEBOOK_IMAGE_VERSION")
    else:
        maia_workspace_base_notebook_image_version = maia_workspace_base_image_version

    if os.environ.get("MAIA_WORKSPACE_BASE_NOTEBOOK_SSH_IMAGE_VERSION") is not None:
        maia_workspace_base_notebook_ssh_image_version = os.environ.get("MAIA_WORKSPACE_BASE_NOTEBOOK_SSH_IMAGE_VERSION")
    else:
        maia_workspace_base_notebook_ssh_image_version = maia_workspace_base_image_version

    if os.environ.get("MAIA_WORKSPACE_IMAGE_VERSION") is not None:
        maia_workspace_image_version = os.environ.get("MAIA_WORKSPACE_IMAGE_VERSION")
    else:
        maia_workspace_image_version = maia_workspace_base_image_version

    if os.environ.get("MAIA_WORKSPACE_NOTEBOOK_IMAGE_VERSION") is not None:
        maia_workspace_notebook_image_version = os.environ.get("MAIA_WORKSPACE_NOTEBOOK_IMAGE_VERSION")
    else:
        maia_workspace_notebook_image_version = maia_workspace_base_image_version

    if os.environ.get("MAIA_WORKSPACE_NOTEBOOK_SSH_IMAGE_VERSION") is not None:
        maia_workspace_notebook_ssh_image_version = os.environ.get("MAIA_WORKSPACE_NOTEBOOK_SSH_IMAGE_VERSION")
    else:
        maia_workspace_notebook_ssh_image_version = maia_workspace_base_image_version

    if os.environ.get("MAIA_WORKSPACE_NOTEBOOK_SSH_ADDONS_IMAGE_VERSION") is not None:
        maia_workspace_notebook_ssh_addons_image_version = os.environ.get("MAIA_WORKSPACE_NOTEBOOK_SSH_ADDONS_IMAGE_VERSION")
    else:
        maia_workspace_notebook_ssh_addons_image_version = maia_workspace_base_image_version

    if os.environ.get("MAIA_WORKSPACE_BASE_NOTEBOOK_SSH_IMAGE_NAME") is not None:
        maia_workspace_base_notebook_ssh_image_name = os.environ.get("MAIA_WORKSPACE_BASE_NOTEBOOK_SSH_IMAGE_NAME")
    else:
        maia_workspace_base_notebook_ssh_image_name = "maia-workspace-base-notebook-ssh"

    if os.environ.get("MAIA_WORKSPACE_NOTEBOOK_SSH_ADDONS_IMAGE_NAME") is not None:
        maia_workspace_notebook_ssh_addons_image_name = os.environ.get("MAIA_WORKSPACE_NOTEBOOK_SSH_ADDONS_IMAGE_NAME")
    else:
        maia_workspace_notebook_ssh_addons_image_name = "maia-workspace-notebook-ssh-addons"

    if os.environ.get("MAIA_LAB_IMAGE_VERSION") is not None:
        maia_lab_image_version = os.environ.get("MAIA_LAB_IMAGE_VERSION")
    else:
        maia_lab_image_version = maia_workspace_base_image_version

    if os.environ.get("MAIA_LAB_PRO_IMAGE_VERSION") is not None:
        maia_lab_pro_image_version = os.environ.get("MAIA_LAB_PRO_IMAGE_VERSION")
    else:
        maia_lab_pro_image_version = maia_workspace_base_image_version

    if os.environ.get("MYSQL_IMAGE_VERSION") is not None:
        mysql_image_version = os.environ.get("MYSQL_IMAGE_VERSION")
    else:
        mysql_image_version = "8.0.28"

    if os.environ.get("MYSQL_IMAGE") is not None:
        mysql_image = os.environ.get("MYSQL_IMAGE")
    else:
        mysql_image = "mysql"

    if os.environ.get("MAIA_ORTHANC_IMAGE") is not None:
        maia_orthanc_image = os.environ.get("MAIA_ORTHANC_IMAGE")
    else:
        maia_orthanc_image = "maia-orthanc"

    if os.environ.get("MAIA_NVFLARE_DASHBOARD_IMAGE") is not None:
        maia_nvflare_dashboard_image_version = os.environ.get("MAIA_NVFLARE_DASHBOARD_IMAGE_VERSION")
    else:
        maia_nvflare_dashboard_image_version = "2.4.0"

    return {
        "maia-kube": maia_kube_image_version,
        "maia-dashboard": maia_dashboard_image_version,
        "maia-dashboard-dev": maia_dashboard_image_version + "-dev",
        "monai-toolkit": monai_toolkit_image_version,
        "maia-xnat": maia_xnat_image_version,
        "maia-orthanc": maia_orthanc_image_version,
        "maia-mlflow": maia_mlflow_image_version,
        "maia-filebrowser": maia_filebrowser_image_version,
        "maia-gpu-booking-admission-controller": maia_gpu_booking_admission_controller_image_version,
        "maia-gpu-booking-pod-terminator": maia_gpu_booking_pod_terminator_image_version,
        "maia-workspace-base": maia_workspace_base_image_version,
        "maia-workspace-base-notebook": maia_workspace_base_notebook_image_version,
        "maia-workspace-base-notebook-ssh": maia_workspace_base_notebook_ssh_image_version,
        "maia-workspace-base-notebook-ssh-image-name": maia_workspace_base_notebook_ssh_image_name,
        "maia-workspace": maia_workspace_image_version,
        "maia-workspace-notebook": maia_workspace_notebook_image_version,
        "maia-workspace-notebook-ssh": maia_workspace_notebook_ssh_image_version,
        "maia-workspace-notebook-ssh-addons": maia_workspace_notebook_ssh_addons_image_version,
        "maia-workspace-notebook-ssh-addons-image-name": maia_workspace_notebook_ssh_addons_image_name,
        "maia-lab": maia_lab_image_version,
        "maia-lab-pro": maia_lab_pro_image_version,
        "mysql": mysql_image_version,
        "mysql_image": mysql_image,
        "maia-orthanc-image": maia_orthanc_image,
        "maia-nvflare-dashboard": maia_nvflare_dashboard_image_version,
    }



#!/usr/bin/env python
from __future__ import annotations

import os
import subprocess
import sys
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

import MAIA
import json
import yaml
import tempfile
from loguru import logger

version = MAIA.__version__

DESC = dedent("""
    Script to install MAIA on a Kubernetes cluster. This script:
    1. Installs the MAIA Ansible collection
    2. Runs MAIA_Configure_Installation.sh to configure the installation
    3. Executes the MAIA installation playbooks in sequence
    """)
EPILOG = dedent("""
    Example call:
    ::
        {filename} --config-folder /path/to/config
    """.format(filename=Path(__file__).stem))


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--config-folder",
        type=str,
        required=True,
        help="Configuration folder where MAIA configuration files will be stored.",
    )

    pars.add_argument(
        "--ansible-collection-path",
        type=str,
        default="git+https://github.com/minnelab/MAIA.git#/ansible/MAIA/Installation",
        help="Path to the MAIA Ansible collection directory. Default: git+https://github.com/minnelab/MAIA.git#/ansible/MAIA/Installation",
    )

    pars.add_argument(
        "--inventory-path",
        type=str,
        default=None,
        help="Path where the inventory file will be created. If not provided, defaults to <config-folder>/inventory",
    )

    pars.add_argument(
        "--skip-configure",
        action="store_true",
        help="Skip running MAIA_Configure_Installation.sh. Use this if configuration is already done.",
    )

    pars.add_argument(
        "--configure-local-host",
        action="store_true",
        help="Configure the local host with the subdomain and IP address for self-signed certificates.",
    )

    pars.add_argument(
        "--configure-no-prompt",
        action="store_true",
        help="Run MAIA_Configure_Installation.sh without prompts (requires env.json to exist).",
    )

    pars.add_argument(
        "--steps",
        nargs="+",
        default=None,
        help="Steps to run. Default: None. If provided, only the specified steps will be run, overriding the steps in the config.yaml file.",
    )

    pars.add_argument("-v", "--version", action="version", version="%(prog)s " + version)

    return pars


def run_command(cmd, check=True, shell=False):
    """Run a shell command and print output."""
    logger.debug(f"Running: {' '.join(cmd) if isinstance(cmd, list) else cmd}")
    if isinstance(cmd, str):
        result = subprocess.run(cmd, shell=True, check=check)
    else:
        result = subprocess.run(cmd, check=check)
    return result


def main():
    parser = get_arg_parser()
    args = parser.parse_args()

    # Resolve paths
    config_folder = Path(args.config_folder).resolve()
    config_folder.mkdir(parents=True, exist_ok=True)

    if Path(config_folder).joinpath("config.yaml").exists():
        config_dict = yaml.safe_load(Path(config_folder).joinpath("config.yaml").read_text())
        os.environ["CONFIG_FOLDER"] = str(config_folder)
        if "env" in config_dict:
            for key, value in config_dict["env"].items():
                if key != "MAIA_PRIVATE_REGISTRY":
                    logger.info(f"Setting environment variable {key} to {value}")
                    if key == "INGRESS_RESOLVER_EMAIL" and value != "":
                        os.environ[key] = value
                    else:
                        os.environ[key] = value
    else:
        config_dict = {}

    if args.steps:
        config_dict["steps"] = args.steps
    playbooks_dir = "maia.installation"

    # Step 1: Install Ansible collection
    logger.info("\n=== Step 1: Installing Ansible collection ===")
    ansible_galaxy_cmd = [
        "ansible-galaxy",
        "collection",
        "install",
        str(args.ansible_collection_path),
    ]
    try:
        run_command(ansible_galaxy_cmd)
    except subprocess.CalledProcessError as e:
        logger.error(f"Error installing Ansible collection: {e}")
        sys.exit(1)

    inventory_path = args.inventory_path if args.inventory_path else config_folder / "inventory"
    # Step 2: Run MAIA_Configure_Installation.sh
    if not args.skip_configure:
        logger.info("\n=== Step 2: Running MAIA_Configure_Installation.sh ===")
        configure_script = "MAIA_Configure_Installation.sh"

        # Check if env.json exists for no-prompt mode
        env_json = config_folder / "env.json"
        if args.configure_no_prompt:
            if not env_json.exists():
                logger.error(f"Error: --configure-no-prompt requires env.json to exist at {env_json}")
                sys.exit(1)
            configure_cmd = [str(configure_script), str(env_json)]
        else:
            # Run with prompts
            configure_cmd = [str(configure_script)]
            if env_json.exists():
                configure_cmd.append(str(env_json))

        try:
            # Run the script interactively (allows prompts)
            subprocess.run(configure_cmd, check=True)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running configuration script: {e}")
            sys.exit(1)
    else:
        logger.info("\n=== Step 2: Skipping MAIA_Configure_Installation.sh ===")

    # Verify config_folder is set (from environment or config)
    if "CONFIG_FOLDER" not in os.environ:
        # Try to get it from env.json if it exists
        env_json = config_folder / "env.json"
        if env_json.exists():
            with open(env_json) as f:
                env_data = json.load(f)
                if "CONFIG_FOLDER" in env_data:
                    os.environ["CONFIG_FOLDER"] = env_data["CONFIG_FOLDER"]
                else:
                    os.environ["CONFIG_FOLDER"] = str(config_folder)
        else:
            os.environ["CONFIG_FOLDER"] = str(config_folder)

    config_folder_env = os.environ["CONFIG_FOLDER"]
    if "cluster_config_extra_env" in config_dict:
        cluster_name = os.environ["CLUSTER_NAME"]
        with open(Path(config_folder_env).joinpath(f"{cluster_name}.yaml"), "r") as f:
            existing_data = yaml.safe_load(f) or {}
            for key, value in config_dict["cluster_config_extra_env"].items():
                existing_data[key] = value
        with open(Path(config_folder_env).joinpath(f"{cluster_name}.yaml"), "w") as f:
            yaml.dump(existing_data, f)
    # Step 3: Run prepare_hosts.yaml
    logger.info("\n=== Step 3: Running prepare_hosts.yaml ===")
    prepare_hosts_cmd = [
        "ansible-playbook",
        "-i",
        str(inventory_path),
        str(playbooks_dir + ".prepare_hosts"),
        "-e",
        f"config_folder={config_folder_env}",
    ]
    if "prepare_hosts" in config_dict:
        for key, value in config_dict["prepare_hosts"].items():
            prepare_hosts_cmd.extend(
                [
                    "-e",
                    f"{key}={value}",
                ]
            )
    if "steps" in config_dict and "prepare_hosts" in config_dict["steps"]:
        try:
            run_command(prepare_hosts_cmd)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running prepare_hosts.yaml: {e}")
            sys.exit(1)
    else:
        logger.info("\n=== Step 3: Skipping prepare_hosts.yaml ===")

    if "selfsigned" in config_dict["cluster_config_extra_env"] and "configure_hosts" in config_dict["steps"]:
        cluster_domain = config_dict["env"]["CLUSTER_DOMAIN"]
        if args.configure_local_host:
            logger.info("\n=== Step 3.1: Running configure_host_linux.yaml for localhost ===")
            target_hosts = "localhost"
            host_ip = "127.0.0.1"
            configure_host_linux_cmd = [
                "ansible-playbook",
                "-K",
                "-i",
                str(inventory_path),
                str(playbooks_dir + ".configure_host"),
                "-e",
                f"target_hosts={target_hosts}",
                "-e",
                f"cluster_domain={cluster_domain}",
                "-e",
                f"host_ip={host_ip}",
            ]
            try:
                run_command(configure_host_linux_cmd)
            except subprocess.CalledProcessError as e:
                logger.error(f"Error running configure_host_linux.yaml: {e}")
                sys.exit(1)
            else:
                logger.info("\n=== Step 3.1: Skipping configure_host_linux.yaml for localhost ===")
        logger.info("\n=== Step 3.2: Running configure_host_linux.yaml for all hosts ===")
        target_hosts = "all"
        configure_host_cmd = [
            "ansible-playbook",
            "-i",
            str(inventory_path),
            str(playbooks_dir + ".configure_host"),
            "-e",
            f"target_hosts={target_hosts}",
            "-e",
            f"cluster_domain={cluster_domain}",
        ]
        try:
            run_command(configure_host_cmd)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running configure_host.yaml: {e}")
            sys.exit(1)
        else:
            logger.info("\n=== Step 3.2: Skipping configure_host.yaml for all hosts ===")
    # Step 4: Run install_k8s_distribution.yaml
    logger.info(f"\n=== Step 4: Running install_{os.environ['K8S_DISTRIBUTION']}.yaml ===")
    install_k8s_distribution_cmd = [
        "ansible-playbook",
        "-i",
        str(inventory_path),
        str(playbooks_dir + f".install_{os.environ['K8S_DISTRIBUTION']}"),
        "-e",
        f"config_folder={config_folder_env}",
    ]
    if f"install_{os.environ['K8S_DISTRIBUTION']}" in config_dict:
        for key, value in config_dict[f"install_{os.environ['K8S_DISTRIBUTION']}"].items():
            install_k8s_distribution_cmd.extend(
                [
                    "-e",
                    f"{key}={value}",
                ]
            )
    if "steps" in config_dict and f"install_{os.environ['K8S_DISTRIBUTION']}" in config_dict["steps"]:
        try:
            run_command(install_k8s_distribution_cmd)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running install_{os.environ['K8S_DISTRIBUTION']}.yaml: {e}")
            sys.exit(1)
    else:
        logger.info(f"\n=== Step 4: Skipping install_{os.environ['K8S_DISTRIBUTION']}.yaml ===")
    # Step 5: Run install_maia_core.yaml
    logger.info("\n=== Step 5: Running install_maia_core.yaml ===")
    install_maia_core_cmd = [
        "ansible-playbook",
        "-i",
        str(inventory_path),
        str(playbooks_dir + ".install_maia_core"),
        "-e",
        f"config_folder={config_folder_env}",
    ]
    if "install_maia_core" in config_dict:
        with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml", delete=False) as tf:
            yaml.dump(config_dict["install_maia_core"], tf)
            vars_file = tf.name

        install_maia_core_cmd.extend(["-e", f"@{vars_file}"])
    if "steps" in config_dict and "install_maia_core" in config_dict["steps"]:
        try:
            run_command(install_maia_core_cmd)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running install_maia_core.yaml: {e}")
            sys.exit(1)
    else:
        logger.info("\n=== Step 5: Skipping install_maia_core.yaml ===")

    # Step 6: Run install_maia_admin.yaml
    logger.info("\n=== Step 6: Running install_maia_admin.yaml ===")
    install_maia_admin_cmd = [
        "ansible-playbook",
        "-i",
        str(inventory_path),
        str(playbooks_dir + ".install_maia_admin"),
        "-e",
        f"config_folder={config_folder_env}",
    ]
    if "install_maia_admin" in config_dict:
        with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml", delete=False) as tf:
            yaml.dump(config_dict["install_maia_admin"], tf)
            vars_file = tf.name

        install_maia_admin_cmd.extend(["-e", f"@{vars_file}"])
    if "steps" in config_dict and "install_maia_admin" in config_dict["steps"]:
        try:
            run_command(install_maia_admin_cmd)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running install_maia_admin.yaml: {e}")
            sys.exit(1)
    else:
        logger.info("\n=== Step 6: Skipping install_maia_admin.yaml ===")

    # Step 7: Run configure_oidc_authentication.yaml
    logger.info("\n=== Step 7: Running configure_oidc_authentication.yaml ===")
    configure_oidc_authentication_cmd = [
        "ansible-playbook",
        "-i",
        str(inventory_path),
        str(playbooks_dir + ".configure_oidc_authentication"),
        "-e",
        f"config_folder={config_folder_env}",
    ]
    if "configure_oidc_authentication" in config_dict:
        for key, value in config_dict["configure_oidc_authentication"].items():
            configure_oidc_authentication_cmd.extend(
                [
                    "-e",
                    f"{key}={value}",
                ]
            )
    if "steps" in config_dict and "configure_oidc_authentication" in config_dict["steps"]:
        try:
            run_command(configure_oidc_authentication_cmd)

        except subprocess.CalledProcessError as e:
            logger.error(f"Error running configure_oidc_authentication.yaml: {e}")
            sys.exit(1)
    else:
        logger.info("\n=== Step 7: Skipping configure_oidc_authentication.yaml ===")

    # Step 8: Run get_kubeconfig_from_rancher_local.yaml
    logger.info("\n=== Step 8: Running get_kubeconfig_from_rancher_local.yaml ===")
    get_kubeconfig_from_rancher_local_cmd = [
        "ansible-playbook",
        "-i",
        str(inventory_path),
        str(playbooks_dir + ".get_kubeconfig_from_rancher_local"),
        "-e",
        f"config_folder={config_folder_env}",
    ]
    if "get_kubeconfig_from_rancher_local" in config_dict:
        for key, value in config_dict["get_kubeconfig_from_rancher_local"].items():
            get_kubeconfig_from_rancher_local_cmd.extend(
                [
                    "-e",
                    f"{key}={value}",
                ]
            )
    if "steps" in config_dict and "get_kubeconfig_from_rancher_local" in config_dict["steps"]:
        try:
            run_command(get_kubeconfig_from_rancher_local_cmd)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running get_kubeconfig_from_rancher_local.yaml: {e}")
            sys.exit(1)
        else:
            logger.info("\n=== Step 8: Skipping get_kubeconfig_from_rancher_local.yaml ===")
    else:
        logger.info("\n=== Step 8: Skipping get_kubeconfig_from_rancher_local.yaml ===")

    # Step 9: Run configure_maia_dashboard.yaml
    logger.info("\n=== Step 9: Running configure_maia_dashboard.yaml ===")
    configure_maia_dashboard_cmd = [
        "ansible-playbook",
        "-i",
        str(inventory_path),
        str(playbooks_dir + ".configure_maia_dashboard"),
        "-e",
        f"config_folder={config_folder_env}",
    ]
    if "configure_maia_dashboard" in config_dict:
        for key, value in config_dict["configure_maia_dashboard"].items():
            configure_maia_dashboard_cmd.extend(
                [
                    "-e",
                    f"{key}={value}",
                ]
            )
    if "steps" in config_dict and "configure_maia_dashboard" in config_dict["steps"]:
        try:
            run_command(configure_maia_dashboard_cmd)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running configure_maia_dashboard.yaml: {e}")
            sys.exit(1)
        else:
            logger.info("\n=== Step 9: Skipping configure_maia_dashboard.yaml ===")
    else:
        logger.info("\n=== Step 9: Skipping configure_maia_dashboard.yaml ===")

    logger.info("\n=== MAIA Installation Complete ===")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
from __future__ import annotations

import asyncio
import datetime
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

import click
import json
import subprocess
from MAIA.kubernetes_utils import create_helm_repo_secret_from_context
import yaml
from hydra import compose as hydra_compose
from hydra import initialize_config_dir
from loguru import logger
from omegaconf import OmegaConf
from pyhelm3 import Client
from MAIA.maia_k8s_distros import get_storage_class, get_ingress_class

import MAIA
from MAIA.maia_admin import (
    create_harbor_values,
    create_keycloak_values,
    create_maia_admin_toolkit_values,
    create_maia_dashboard_values,
    install_maia_project,
    create_rancher_values,
)

version = MAIA.__version__


TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent("""
    Script to Install MAIA Admin Toolkit to a Kubernetes cluster from ArgoCD. The specific MAIA configuration
    is specified by setting the corresponding ``--maia-config-file``, and the cluster configuration is
    specified by setting the corresponding ``--cluster-config``.
    """)  # noqa: E501
EPILOG = dedent("""
    Example call:
    ::
        {filename} --cluster-config /PATH/TO/cluster_config.yaml --config-folder /PATH/TO/config_folder
    """.format(filename=Path(__file__).stem))  # noqa: E501


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--cluster-config",
        type=str,
        required=True,
        help="YAML configuration file used to extract the cluster configuration.",
    )

    pars.add_argument(
        "--config-folder",
        type=str,
        required=True,
        help="Configuration Folder where to locate (and temporarily store) the MAIA configuration files.",
    )

    pars.add_argument("-v", "--version", action="version", version="%(prog)s " + version)

    return pars


async def verify_installed_maia_admin_toolkit(project_id, namespace):
    logger.info(f"KUBECONFIG: {os.environ['KUBECONFIG']}")
    client = Client(kubeconfig=os.environ["KUBECONFIG"])

    try:
        revision = await client.get_current_revision(project_id, namespace=namespace)
    except Exception as e:
        logger.error(f"An error occurred: {e}")
        return -1
    chart_metadata = await revision.chart_metadata()
    logger.info(
        f"Release: {revision.release.name}, Namespace: {revision.release.namespace}, "
        f"Revision: {revision.revision}, Status: {revision.status}, "
        f"Chart: {chart_metadata.name}, Version: {chart_metadata.version}"
    )
    return revision.revision


@click.command()
@click.option("--cluster-config", type=str)
@click.option("--config-folder", type=str)
def main(cluster_config, config_folder):
    install_maia_admin_toolkit(cluster_config, config_folder)


def install_maia_admin_toolkit(cluster_config, config_folder):
    cluster_config_dict = yaml.safe_load(Path(cluster_config).read_text())
    private_maia_registry = os.environ.get("MAIA_PRIVATE_REGISTRY", None)
    admin_group_id = os.environ["admin_group_ID"]
    project_id = "maia-admin"

    cluster_address = "https://kubernetes.default.svc"  # TODO: Change this to make it configurable

    if "ingress_class" not in cluster_config_dict:
        if "k8s_distribution" in cluster_config_dict:
            cluster_config_dict["ingress_class"] = get_ingress_class(cluster_config_dict["k8s_distribution"])

    if "storage_class" not in cluster_config_dict:
        if "k8s_distribution" in cluster_config_dict:
            cluster_config_dict["storage_class"] = get_storage_class(cluster_config_dict["k8s_distribution"])

    helm_commands = []

    helm_commands.append(create_harbor_values(config_folder, project_id, cluster_config_dict))
    helm_commands.append(create_keycloak_values(config_folder, project_id, cluster_config_dict))
    helm_commands.append(create_rancher_values(config_folder, project_id, cluster_config_dict))
    helm_commands.append(create_maia_admin_toolkit_values(config_folder, project_id, cluster_config_dict))
    helm_commands.append(create_maia_dashboard_values(config_folder, project_id, cluster_config_dict))

    json_key_path = os.environ.get("JSON_KEY_PATH", None)
    for helm_command in helm_commands:
        if (
            not helm_command["repo"].startswith("http") and not Path(helm_command["repo"]).exists()
        ):  # If the repo is not a HTTP URL, it is an OCI registry (i.e. Harbor)
            original_repo = helm_command["repo"]
            helm_command["repo"] = f"oci://{helm_command['repo']}"
            try:
                with open(json_key_path, "r") as f:
                    docker_credentials = json.load(f)
                    username = docker_credentials.get("username")
                    password = docker_credentials.get("password")
            except Exception:
                with open(json_key_path, "r") as f:
                    docker_credentials = f.read()
                    username = "_json_key"
                    password = docker_credentials

            subprocess.run(
                [
                    "helm",
                    "registry",
                    "login",
                    original_repo,
                    "--username",
                    username,
                    "--password-stdin",
                ],
                input=password.encode(),
                check=True,
            )
            logger.debug(
                " ".join(
                    [
                        "helm",
                        "registry",
                        "login",
                        original_repo,
                        "--username",
                        username,
                        "--password-stdin",
                    ]
                )
            )
            subprocess.run(
                [
                    "helm",
                    "pull",
                    helm_command["repo"] + "/" + helm_command["chart"],
                    "--version",
                    helm_command["version"],
                    "--destination",
                    "/tmp",
                ]
            )
            logger.debug(
                " ".join(
                    [
                        "helm",
                        "pull",
                        helm_command["repo"] + "/" + helm_command["chart"],
                        "--version",
                        helm_command["version"],
                        "--destination",
                        "/tmp",
                    ]
                )
            )
            cmd = [
                "helm",
                "upgrade",
                "--install",
                # "--wait",
                "-n",
                helm_command["namespace"],
                helm_command["release"],
                "/tmp/" + helm_command["chart"] + "-" + helm_command["version"] + ".tgz",
                "--values",
                helm_command["values"],
            ]
            logger.debug(" ".join(cmd))
        else:
            cmd = [
                "helm",
                "upgrade",
                "--install",
                "--wait",
                "-n",
                helm_command["namespace"],
                helm_command["release"],
                helm_command["chart"],
                "--repo",
                helm_command["repo"],
                "--version",
                helm_command["version"],
                "--values",
                helm_command["values"],
            ]
            logger.debug(f"Helm command: {' '.join(cmd)}")

    values = {
        "defaults": [
            "_self_",
            {"harbor_values": "harbor_values"},
            {"keycloak_values": "keycloak_values"},
            {"maia_admin_toolkit_values": "maia_admin_toolkit_values"},
            {"maia_dashboard_values": "maia_dashboard_values"},
            {"rancher_values": "rancher_values"},
        ],
        "argo_namespace": os.environ["argocd_namespace"],
        "admin_group_ID": admin_group_id,
        "destination_server": f"{cluster_address}",
        "sourceRepos": [
            "https://minnelab.github.io/MAIA/",
            "https://github.com/minnelab/MAIA.git",
            "https://helm.goharbor.io",
            "https://charts.bitnami.com/bitnami",
            "https://releases.rancher.com/server-charts/latest",
        ],
    }
    Path(config_folder).joinpath(project_id).mkdir(parents=True, exist_ok=True)

    with open(Path(config_folder).joinpath(project_id, "values.yaml"), "w") as f:
        f.write(OmegaConf.to_yaml(values))

    initialize_config_dir(config_dir=str(Path(config_folder).joinpath(project_id)), job_name=project_id)
    cfg = hydra_compose("values.yaml")
    OmegaConf.save(
        cfg,
        str(Path(config_folder).joinpath(project_id, f"{project_id}_values.yaml")),
        resolve=True,
    )

    revision = asyncio.run(verify_installed_maia_admin_toolkit(project_id, os.environ["argocd_namespace"]))

    if json_key_path is not None:
        try:
            with open(json_key_path, "r") as f:
                docker_credentials = json.load(f)
                username = docker_credentials.get("username")
                password = docker_credentials.get("password")
        except Exception:
            with open(json_key_path, "r") as f:
                docker_credentials = f.read()
                username = "_json_key"
                password = docker_credentials
        create_helm_repo_secret_from_context(
            repo_name=f"maia-private-registry-{project_id}",
            argocd_namespace=os.environ["argocd_namespace"],
            helm_repo_config={
                "username": username,
                "password": password,
                "project": project_id,
                "url": private_maia_registry,
                "type": "helm",
                "name": f"maia-private-registry-{project_id}",
                "enableOCI": "true",
            },
        )
    if revision == -1:
        logger.info("Installing MAIA Admin Toolkit")

        project_chart = os.environ["admin_project_chart"]
        project_repo = os.environ["admin_project_repo"]
        project_version = os.environ["admin_project_version"]
        asyncio.run(
            install_maia_project(
                project_id,
                Path(config_folder).joinpath(project_id, f"{project_id}_values.yaml"),
                os.environ["argocd_namespace"],
                project_chart,
                project_repo=project_repo,
                project_version=project_version,
                json_key_path=json_key_path,
            )
        )
    else:
        logger.info("Upgrading MAIA Admin Toolkit")

        project_chart = os.environ["admin_project_chart"]
        project_repo = os.environ["admin_project_repo"]
        project_version = os.environ["admin_project_version"]
        asyncio.run(
            install_maia_project(
                project_id,
                Path(config_folder).joinpath(project_id, f"{project_id}_values.yaml"),
                os.environ["argocd_namespace"],
                project_chart,
                project_repo=project_repo,
                project_version=project_version,
                json_key_path=json_key_path,
            )
        )


if __name__ == "__main__":
    main()



from __future__ import annotations

import asyncio
import json
import os
import subprocess
from pathlib import Path
import requests
from loguru import logger
import yaml
from omegaconf import OmegaConf
from pyhelm3 import Client

from MAIA.maia_fn import generate_human_memorable_password
from MAIA.maia_k8s_distros import get_api_port
from MAIA.versions import (
    define_maia_admin_versions,
    define_maia_project_versions,
    define_docker_image_versions,
)

maia_workspace_notebook_ssh_addons_image_version = define_docker_image_versions()["maia-workspace-notebook-ssh-addons"]
maia_workspace_notebook_ssh_addons_image_name = define_docker_image_versions()["maia-workspace-notebook-ssh-addons-image-name"]
maia_workspace_base_notebook_ssh_image_version = define_docker_image_versions()["maia-workspace-base-notebook-ssh"]
maia_workspace_base_notebook_ssh_image_name = define_docker_image_versions()["maia-workspace-base-notebook-ssh-image-name"]
maia_project_chart_version = define_maia_project_versions()["maia_project_chart_version"]
maia_orthanc_image_version = define_docker_image_versions()["maia-orthanc"]
admin_toolkit_chart_version = define_maia_admin_versions()["admin_toolkit_chart_version"]
admin_toolkit_chart_type = define_maia_admin_versions()["admin_toolkit_chart_type"]
rancher_chart_version = define_maia_admin_versions()["rancher_chart_version"]
harbor_chart_version = define_maia_admin_versions()["harbor_chart_version"]
keycloak_chart_version = define_maia_admin_versions()["keycloak_chart_version"]
maia_dashboard_chart_version = define_maia_admin_versions()["maia_dashboard_chart_version"]
maia_dashboard_image_version = define_maia_admin_versions()["maia_dashboard_image_version"]
maia_dashboard_dev_tag_suffix = define_maia_admin_versions()["maia_dashboard_dev_tag_suffix"]
maia_dashboard_chart_type = define_maia_admin_versions()["maia_dashboard_chart_type"]
mysql_image = define_docker_image_versions()["mysql_image"]
mysql_image_version = define_docker_image_versions()["mysql"]


def get_maia_toolkit_apps(group_id, password, argo_cd_host):
    """
    Retrieve and print information about a specific project and its associated applications from Argo CD.

    Parameters
    ----------
    group_id : str
        The group identifier used to construct project and application names.
    password : str
        The authorization token for accessing the Argo CD API.
    argo_cd_host : str
        The host URL of the Argo CD server.

    Returns
    -------
    list
        A list of dictionaries containing the name and version of each application.
        Each dictionary has the following keys:
        - name (str): The name of the application.
        - version (str): The version of the application.

    Example
    -------
    apps = get_maia_toolkit_apps("maia-core", "password", "http://localhost:8080")
    logger.info(f"Apps: {apps}")

    """

    response = requests.post(f"{argo_cd_host}/api/v1/session", json={"username": "admin", "password": password}, verify=False)
    if response.status_code == 200:
        cookies = {"argocd.token": response.json()["token"]}  # <- session cookie
    else:
        logger.error(f"Failed to get token: {response.status_code}")
        logger.error(f"Response: {response.text}")
        return

    apps_url = f"{argo_cd_host}/api/v1/applications?projects={group_id}"
    resp = requests.get(apps_url, cookies=cookies, verify=False)

    apps = []
    if resp.status_code == 200:
        data = resp.json()
        if "items" in data and data["items"] is not None:
            app_names = [app["metadata"]["name"] for app in data.get("items", [])]
            logger.info(f"✅ Applications in project: {group_id}")
            for name in app_names:
                logger.info(f" - {name}")
                item = next((item for item in data.get("items", []) if item["metadata"]["name"] == name), None)
                apps.append(
                    {
                        "name": name,
                        "version": item["spec"]["source"]["targetRevision"],
                        "repo": item["spec"]["source"]["repoURL"],
                    }
                )
                if "chart" in item["spec"]["source"]:
                    apps[-1]["chart"] = item["spec"]["source"]["chart"]
                elif "path" in item["spec"]["source"]:
                    apps[-1]["path"] = item["spec"]["source"]["path"]
        return apps
    else:
        logger.error(f"❌ Failed to fetch apps: {resp.status_code}")
        logger.error(f"Response: {resp.text}")
        return []


async def install_maia_project(
    group_id, values_file, argo_cd_namespace, project_chart, project_repo=None, project_version=None, json_key_path=None
):
    """
    Installs or upgrades a MAIA project using the specified Helm chart and values file.

    Parameters
    ----------
    group_id : str
        The group ID for the project. This will be used as the release name.
    values_file : str
        Path to the YAML file containing the values for the Helm chart.
    argo_cd_namespace : str
        The namespace in which to install the project.
    project_chart : str
        The name of the Helm chart to use for the project.
    project_repo : str, optional
        The repository URL where the Helm chart is located. Defaults to None.
    project_version : str, optional
        The version of the Helm chart to use. Defaults to None.
    json_key_path : str, optional
        Path to the JSON key file for authentication with the Helm registry. Defaults to None.

    Returns
    -------
    None

    Raises
    ------
    FileNotFoundError
        If the values file does not exist.
    yaml.YAMLError
        If there is an error parsing the values file.
    Exception
        If there is an error during the installation or upgrade process.
    """
    client = Client(kubeconfig=os.environ["KUBECONFIG"])
    chart_name = group_id.lower().replace("_", "-")
    if chart_name[-1] == "-":
        chart_name = chart_name[:-1]
    if chart_name[0] == "-":
        chart_name = chart_name[1:]

    if not project_repo.startswith("http") and not Path(project_repo).exists() and not project_repo.startswith("git+"):
        chart = str("/tmp/" + project_chart + "-" + project_version + ".tgz")
        project_chart = "oci://" + project_repo + "/" + project_chart

        try:
            try:
                with open(json_key_path, "r") as f:
                    docker_credentials = json.load(f)
                    username = docker_credentials.get("username")
                    password = docker_credentials.get("password")
            except Exception:
                with open(json_key_path, "r") as f:
                    docker_credentials = f.read()
                    username = "_json_key"
                    password = docker_credentials
            logger.debug(f"helm registry login {project_repo} --insecure -u {username} --password-stdin")
            result = subprocess.run(
                ["helm", "registry", "login", project_repo, "--insecure", "-u", username, "--password-stdin"],
                input=password.encode(),
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
            )
            logger.info("✅ Helm registry login successful.")
            logger.debug(result.stdout.decode())
        except subprocess.CalledProcessError as e:
            logger.error("❌ Helm registry login failed.")
            logger.error("STDOUT:", e.stdout.decode())
            logger.error("STDERR:", e.stderr.decode())
            await asyncio.sleep(1)
            return "Deployment failed: Helm registry login failed."
        subprocess.run(
            ["helm", "pull", project_chart, "-d", "/tmp", "--insecure-skip-tls-verify", "--version", project_version], check=True
        )
        subprocess.run(
            [
                "helm",
                "upgrade",
                "--install",
                chart_name,
                chart,
                "--namespace",
                argo_cd_namespace,
                "--values",
                str(values_file),
                "--wait",
            ],
            check=True,
        )
        await asyncio.sleep(1)
        return ""
    if Path(project_repo).exists():
        chart = await client.get_chart(project_repo, version=project_version)
    elif project_repo.startswith("git+"):
        ...
    elif not project_repo.startswith("http"):
        chart = await client.get_chart(project_chart, repo=project_repo, version=project_version, insecure=True)
    else:
        chart = await client.get_chart(project_chart, repo=project_repo, version=project_version)
    with open(values_file) as f:
        values = yaml.safe_load(f)

    if project_repo.startswith("git+"):
        subprocess.run(
            [
                "helm",
                "upgrade",
                "--install",
                chart_name,
                project_repo,
                "--namespace",
                argo_cd_namespace,
                "--values",
                str(values_file),
                "--wait",
            ],
            check=True,
        )
    else:
        revision = await client.install_or_upgrade_release(chart_name, chart, values, namespace=argo_cd_namespace, wait=True)
        logger.debug(revision.release.name, revision.release.namespace, revision.revision, str(revision.status))

    return ""


def create_maia_admin_toolkit_values(config_folder, project_id, cluster_config_dict):
    """
    Creates and writes the MAIA admin toolkit values to a YAML file.

    Parameters
    ----------
    config_folder : str
        The path to the configuration folder.
    project_id : str
        The project identifier.
    cluster_config_dict : dict
        Dictionary containing cluster configuration values.

    Returns
    -------
    dict
        A dictionary containing the namespace, release name, chart name, repository URL, chart version,
        and the path to the generated values YAML file.
    """
    admin_group_id = os.environ["admin_group_ID"]

    admin_toolkit_values = {
        "namespace": "maia-admin-toolkit",
        "chart_version": admin_toolkit_chart_version,
    }

    if "ARGOCD_DISABLED" in os.environ and os.environ["ARGOCD_DISABLED"] == "True" and admin_toolkit_chart_type == "git_repo":
        raise ValueError("ARGOCD_DISABLED is set to True and core_toolkit_chart_type is set to git_repo, which is not allowed")

    if admin_toolkit_chart_type == "helm_repo":
        admin_toolkit_values["repo_url"] = os.environ.get("MAIA_PRIVATE_REGISTRY", "https://minnelab.github.io/MAIA/")
        admin_toolkit_values["chart_name"] = "maia-admin-toolkit"
    elif admin_toolkit_chart_type == "git_repo":
        admin_toolkit_values["repo_url"] = os.environ.get("MAIA_PRIVATE_REGISTRY", "https://github.com/minnelab/MAIA.git")
        admin_toolkit_values["path"] = "charts/maia-admin-toolkit"

    admin_toolkit_values.update(
        {
            "argocd": {
                "enabled": True,
                "argocd_namespace": "argocd",
                "argocd_domain": "argocd." + cluster_config_dict["domain"],
                "keycloak_issuer_url": "https://iam." + cluster_config_dict["domain"] + "/realms/maia",
                "keycloak_client_id": "maia",
                "keycloak_client_secret": os.environ["keycloak_client_secret"],
            },
            "admin_group_ID": admin_group_id,
            "harbor": {
                "enabled": True,
                "values": {"namespace": "harbor", "storageClassName": cluster_config_dict["storage_class"]},
            },
            "keycloak": {
                "enabled": True,
                "values": {"namespace": "keycloak", "storageClassName": cluster_config_dict["storage_class"]},
            },
            "minio": {
                "enabled": True,
                "namespace": "maia-dashboard",
                "adminAccessKey": "maia-admin",
                "adminSecretKey": os.environ["minio_admin_password"],
                "image": "quay.io/minio/minio:RELEASE.2025-04-08T15-41-24Z",
                "storageSize": "10Gi",
                "storageClassName": cluster_config_dict["storage_class"],
                "consoleDomain": "minio." + cluster_config_dict["domain"],
                "apiDomain": "minio-api." + cluster_config_dict["domain"],
                "rootAccessKey": "root",
                "rootSecretKey": os.environ["minio_root_password"],
                "openIdClientId": "maia",
                "openIdClientSecret": os.environ["keycloak_client_secret"],
                "openIdConfigUrl": "https://iam."
                + cluster_config_dict["domain"]
                + "/realms/maia/.well-known/openid-configuration",
                "ingress": {
                    "annotations": {},
                },
            },
        }
    )

    if "externalCA" in cluster_config_dict:
        admin_toolkit_values["minio"]["externalCA"] = {}
        admin_toolkit_values["minio"]["externalCA"]["name"] = cluster_config_dict["externalCA"]["name"]
        admin_toolkit_values["minio"]["externalCA"]["cert"] = open(Path(cluster_config_dict["externalCA"]["cert"])).read()
        admin_toolkit_values["argocd"]["rootCA"] = open(Path(cluster_config_dict["externalCA"]["cert"])).read()

    if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
        admin_toolkit_values["argocd"]["rootCA"] = open(Path(cluster_config_dict["rootCA"])).read()

    if cluster_config_dict["ingress_class"] == "maia-core-traefik":
        admin_toolkit_values["minio"]["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.entrypoints"] = "websecure"
        admin_toolkit_values["minio"]["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.tls"] = "true"
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            ...
        else:
            admin_toolkit_values["minio"]["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.tls.certresolver"] = (
                cluster_config_dict["traefik_resolver"]
            )
            admin_toolkit_values["certResolver"] = cluster_config_dict["traefik_resolver"]
    elif cluster_config_dict["ingress_class"] == "nginx":
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            admin_toolkit_values["minio"]["ingress"]["annotations"]["cert-manager.io/cluster-issuer"] = "kubernetes-ca-issuer"
        else:
            admin_toolkit_values["minio"]["ingress"]["annotations"]["cert-manager.io/cluster-issuer"] = "cluster-issuer"
        admin_toolkit_values["minio"]["ingress"]["tlsSecretName"] = f"{project_id}-tls"
        admin_toolkit_values["minio"]["ingress"]["tlsSecretNameApi"] = f"{project_id}-tls-api"

    Path(config_folder).joinpath(project_id, "maia_admin_toolkit_values").mkdir(parents=True, exist_ok=True)

    with open(Path(config_folder).joinpath(project_id, "maia_admin_toolkit_values", "maia_admin_toolkit_values.yaml"), "w") as f:
        f.write(OmegaConf.to_yaml(admin_toolkit_values))

    return {
        "namespace": admin_toolkit_values["namespace"],
        "release": f"{project_id}-toolkit",
        "chart": admin_toolkit_values["chart_name"] if admin_toolkit_chart_type == "helm_repo" else admin_toolkit_values["path"],
        "repo": admin_toolkit_values["repo_url"],
        "version": admin_toolkit_values["chart_version"],
        "values": str(Path(config_folder).joinpath(project_id, "maia_admin_toolkit_values", "maia_admin_toolkit_values.yaml")),
    }


def create_harbor_values(config_folder, project_id, cluster_config_dict):
    """
    Create and save Harbor values configuration for a given project and cluster configuration.

    Parameters
    ----------
    config_folder : str
        The path to the configuration folder where the Harbor values file will be saved.
    project_id : str
        The unique identifier for the project.
    cluster_config_dict : dict
        A dictionary containing cluster configuration details, including:
            - domain (str): The domain name for the Harbor registry.
            - ingress_class (str): The ingress class to be used (e.g., "maia-core-traefik", "nginx").
            - traefik_resolver (str, optional): The Traefik resolver to be used if ingress_class is "maia-core-traefik".

    Returns
    -------
    dict
        A dictionary containing the following keys:
        - namespace (str): The Kubernetes namespace for Harbor.
        - release (str): The release name for the Harbor Helm chart.
        - chart (str): The name of the Harbor Helm chart.
        - repo (str): The URL of the Harbor Helm chart repository.
        - version (str): The version of the Harbor Helm chart.
        - values (str): The path to the generated Harbor values YAML file.
    """
    domain = cluster_config_dict["domain"]
    harbor_values = {
        "namespace": "harbor",
        "repo_url": "https://helm.goharbor.io",
        "chart_name": "harbor",
        "chart_version": harbor_chart_version,
    }

    harbor_values.update(
        {
            "expose": {
                "type": "ingress",
                "tls": {"enabled": True},
                "ingress": {
                    "hosts": {"core": f"registry.{domain}"},
                    "annotations": {},
                    "controller": "default",
                    "className": cluster_config_dict["ingress_class"],
                },
            },
            "externalURL": f"https://registry.{domain}",
            "persistence": {
                "enabled": True,
                "resourcePolicy": "keep",
                "persistentVolumeClaim": {
                    "registry": {
                        "existingClaim": "pvc-harbor",
                        "subPath": "registry",
                        "storageClass": cluster_config_dict["storage_class"],
                        "accessMode": "ReadWriteMany",
                    },
                    "jobservice": {
                        "jobLog": {
                            "existingClaim": "pvc-harbor",
                            "subPath": "job_logs",
                            "storageClass": cluster_config_dict["storage_class"],
                            "accessMode": "ReadWriteMany",
                        }
                    },
                    "database": {
                        "existingClaim": "pvc-harbor",
                        "subPath": "database",
                        "storageClass": cluster_config_dict["storage_class"],
                        "accessMode": "ReadWriteMany",
                    },
                    "redis": {
                        "existingClaim": "pvc-harbor",
                        "subPath": "redis",
                        "storageClass": cluster_config_dict["storage_class"],
                        "accessMode": "ReadWriteMany",
                    },
                    "trivy": {
                        "existingClaim": "pvc-harbor",
                        "subPath": "trivy",
                        "storageClass": cluster_config_dict["storage_class"],
                        "accessMode": "ReadWriteMany",
                    },
                },
                "imageChartStorage": {"type": "filesystem"},
            },
            "database": {"internal": {"password": "harbor"}},
            "metrics": {
                "enabled": True,
                "core": {"path": "/metrics", "port": 8001},
                "registry": {"path": "/metrics", "port": 8001},
                "jobservice": {"path": "/metrics", "port": 8001},
                "exporter": {"path": "/metrics", "port": 8001},
            },
        }
    )

    if cluster_config_dict["ingress_class"] == "maia-core-traefik":
        harbor_values["expose"]["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.entrypoints"] = "websecure"
        harbor_values["expose"]["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.tls"] = "true"
        harbor_values["expose"]["tls"]["certSource"] = "none"
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            harbor_values["expose"]["ingress"]["annotations"][
                "traefik.ingress.kubernetes.io/router.tls.certresolver"
            ] = "kubernetes-ca-issuer"
        else:
            harbor_values["expose"]["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.tls.certresolver"] = (
                cluster_config_dict["traefik_resolver"]
            )
    elif cluster_config_dict["ingress_class"] == "nginx":
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            ...
        else:
            harbor_values["expose"]["ingress"]["annotations"]["cert-manager.io/cluster-issuer"] = "cluster-issuer"

    Path(config_folder).joinpath(project_id, "harbor_values").mkdir(parents=True, exist_ok=True)
    with open(Path(config_folder).joinpath(project_id, "harbor_values", "harbor_values.yaml"), "w") as f:
        f.write(OmegaConf.to_yaml(harbor_values))

    return {
        "namespace": harbor_values["namespace"],
        "release": f"{project_id}-harbor",
        "chart": harbor_values["chart_name"],
        "repo": harbor_values["repo_url"],
        "version": harbor_values["chart_version"],
        "values": str(Path(config_folder).joinpath(project_id, "harbor_values", "harbor_values.yaml")),
    }


def create_keycloak_values(config_folder, project_id, cluster_config_dict):
    """
    Generates Keycloak Helm chart values and writes them to a YAML file.

    Parameters
    ----------
    config_folder : str
        The path to the configuration folder where the YAML file will be saved.
    project_id : str
        The project identifier used to create a unique namespace and release name.
    cluster_config_dict : dict
        A dictionary containing cluster configuration details such as domain, ingress class, and traefik resolver.

    Returns
    -------
    dict
        A dictionary containing the namespace, release name, chart name, repository URL, chart version,
        and the path to the generated values YAML file.
    """
    keycloak_values = {
        "namespace": "keycloak",
        "repo_url": "https://charts.bitnami.com/bitnami",
        "chart_name": "keycloak",
        "chart_version": keycloak_chart_version,
    }

    keycloak_values.update(
        {
            "extraEnvVars": [
                {"name": "KEYCLOAK_EXTRA_ARGS", "value": "--import-realm"},
                {"name": "PROXY_ADDRESS_FORWARDING", "value": "true"},
                {"name": "KEYCLOAK_HOSTNAME", "value": "iam." + cluster_config_dict["domain"]},
            ],
            "proxy": "edge",
            "ingress": {
                "enabled": True,
                "tls": True,
                "ingressClassName": cluster_config_dict["ingress_class"],
                "hostname": "iam." + cluster_config_dict["domain"],
                "annotations": {},
            },
            "auth": {
                "adminPassword": os.environ.get("keycloak_admin_password", ""),
            },
            "extraVolumeMounts": [
                {
                    "name": "keycloak-import",
                    "mountPath": "/opt/bitnami/keycloak/data/import",
                },
                {"name": "keycloak-themes", "mountPath": "/opt/bitnami/keycloak/themes"},
            ],
            "extraVolumes": [
                {"name": "keycloak-import", "configMap": {"name": "maia-realm-import"}},
                {"name": "keycloak-themes", "persistentVolumeClaim": {"claimName": "pvc-keycloak-themes"}},
            ],
        }
    )

    if cluster_config_dict["ingress_class"] == "maia-core-traefik":
        keycloak_values["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.entrypoints"] = "websecure"
        keycloak_values["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.tls"] = "true"
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            ...
        else:
            keycloak_values["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.tls.certresolver"] = (
                cluster_config_dict["traefik_resolver"]
            )
    elif cluster_config_dict["ingress_class"] == "nginx":
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            keycloak_values["ingress"]["annotations"]["cert-manager.io/cluster-issuer"] = "kubernetes-ca-issuer"
        else:
            keycloak_values["ingress"]["annotations"]["cert-manager.io/cluster-issuer"] = "cluster-issuer"

    Path(config_folder).joinpath(project_id, "keycloak_values").mkdir(parents=True, exist_ok=True)
    with open(Path(config_folder).joinpath(project_id, "keycloak_values", "keycloak_values.yaml"), "w") as f:
        f.write(OmegaConf.to_yaml(keycloak_values))

    return {
        "namespace": keycloak_values["namespace"],
        "release": f"{project_id}-keycloak",
        "chart": keycloak_values["chart_name"],
        "repo": keycloak_values["repo_url"],
        "version": keycloak_values["chart_version"],
        "values": str(Path(config_folder).joinpath(project_id, "keycloak_values", "keycloak_values.yaml")),
    }


def create_maia_dashboard_values(config_folder, project_id, cluster_config_dict):
    """
    Create MAIA dashboard values for Helm chart deployment.

    Parameters
    ----------
    config_folder : str
        The path to the configuration folder.
    project_id : str
        The project identifier.
    cluster_config_dict : dict
        Dictionary containing cluster configuration details.

    Returns
    -------
    dict
        A dictionary containing the namespace, release name, chart name, repository URL, chart version,
        and the path to the generated values YAML file.
    """
    maia_dashboard_values = {
        "namespace": "maia-dashboard",
        "chart_version": maia_dashboard_chart_version,
    }

    if "ARGOCD_DISABLED" in os.environ and os.environ["ARGOCD_DISABLED"] == "True" and maia_dashboard_chart_type == "git_repo":
        raise ValueError("ARGOCD_DISABLED is set to True and maia_dashboard_chart_type is set to git_repo, which is not allowed")

    if maia_dashboard_chart_type == "helm_repo":
        maia_dashboard_values["repo_url"] = os.environ.get("MAIA_PRIVATE_REGISTRY", "https://minnelab.github.io/MAIA/")
        maia_dashboard_values["chart_name"] = "maia-dashboard"
    elif maia_dashboard_chart_type == "git_repo":
        maia_dashboard_values["repo_url"] = os.environ.get("MAIA_PRIVATE_REGISTRY", "https://github.com/minnelab/MAIA.git")
        maia_dashboard_values["path"] = "charts/maia-dashboard"

    default_registry = os.environ.get("MAIA_REGISTRY", "ghcr.io/minnelab")
    maia_dashboard_values.update(
        {
            "ingress": {
                "enabled": True,
                "className": cluster_config_dict["ingress_class"],
                "annotations": {},
                "hosts": [
                    {
                        "host": "maia." + cluster_config_dict["domain"],
                        "paths": [
                            {"path": "/", "pathType": "ImplementationSpecific"},
                            {"path": "/maia-api/", "pathType": "ImplementationSpecific"},
                            {"path": "/maia/", "pathType": "ImplementationSpecific"},
                        ],
                    }
                ],
                "tls": [{"hosts": ["maia." + cluster_config_dict["domain"]]}],
            },
            "env": [
                {"name": "DEBUG", "value": "False"},
                {"name": "SERVER", "value": "maia." + cluster_config_dict["domain"]},
                {"name": "LOCAL_DB_PATH", "value": "/etc/MAIA-Dashboard/db"},
            ],
            "storageClass": cluster_config_dict["storage_class"],
            "image": {"repository": f"{default_registry}/maia-dashboard", "tag": maia_dashboard_image_version},
            "dashboard": {"local_config_path": "/mnt/dashboard-config"},
            "mysql": {"enabled": True, "image": mysql_image, "tag": mysql_image_version},
        }
    )

    if cluster_config_dict["ingress_class"] == "maia-core-traefik":
        maia_dashboard_values["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.entrypoints"] = "websecure"
        maia_dashboard_values["ingress"]["annotations_redirect"] = {"traefik.ingress.kubernetes.io/router.entrypoints": "web"}
        maia_dashboard_values["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.tls"] = "true"
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            ...
        else:
            maia_dashboard_values["ingress"]["annotations"]["traefik.ingress.kubernetes.io/router.tls.certresolver"] = (
                cluster_config_dict["traefik_resolver"]
            )
    elif cluster_config_dict["ingress_class"] == "nginx":
        maia_dashboard_values["ingress"]["annotations"]["nginx.ingress.kubernetes.io/proxy-body-size"] = "8g"
        maia_dashboard_values["ingress"]["annotations"]["nginx.ingress.kubernetes.io/proxy-read-timeout"] = "300"
        maia_dashboard_values["ingress"]["annotations"]["nginx.ingress.kubernetes.io/proxy-send-timeout"] = "300"
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            maia_dashboard_values["ingress"]["annotations"]["cert-manager.io/cluster-issuer"] = "kubernetes-ca-issuer"
        else:
            maia_dashboard_values["ingress"]["annotations"]["cert-manager.io/cluster-issuer"] = "cluster-issuer"
        maia_dashboard_values["ingress"]["tls"][0]["secretName"] = cluster_config_dict["domain"]

    # Authentication
    maia_dashboard_values["env"].extend(
        [
            {"name": "OIDC_RP_CLIENT_ID", "value": "maia"},
            {"name": "OIDC_RP_PUBLIC_CLIENT_ID", "value": "maia-public"},
            {"name": "OIDC_RP_CLIENT_SECRET", "value": os.environ["keycloak_client_secret"]},
            {"name": "OIDC_SERVER_URL", "value": "https://iam." + cluster_config_dict["domain"]},
            {"name": "OIDC_REALM_NAME", "value": "maia"},
            {"name": "OIDC_USERNAME", "value": "admin"},
            {"name": "OIDC_ISSUER_URL", "value": "https://iam." + cluster_config_dict["domain"] + "/realms/maia"},
            {
                "name": "OIDC_OP_AUTHORIZATION_ENDPOINT",
                "value": "https://iam." + cluster_config_dict["domain"] + "/realms/maia/protocol/openid-connect/auth",
            },
            {
                "name": "OIDC_OP_TOKEN_ENDPOINT",
                "value": "https://iam." + cluster_config_dict["domain"] + "/realms/maia/protocol/openid-connect/token",
            },
            {
                "name": "OIDC_OP_USER_ENDPOINT",
                "value": "https://iam." + cluster_config_dict["domain"] + "/realms/maia/protocol/openid-connect/userinfo",
            },
            {
                "name": "OIDC_OP_JWKS_ENDPOINT",
                "value": "https://iam." + cluster_config_dict["domain"] + "/realms/maia/protocol/openid-connect/certs",
            },
            {"name": "OIDC_RP_SIGN_ALGO", "value": "RS256"},
            {"name": "OIDC_RP_SCOPES", "value": "openid email profile"},
        ]
    )

    # Cluster Access
    maia_dashboard_values["clusters"] = [
        {
            "api": "https://mgmt." + cluster_config_dict["domain"] + "/k8s/clusters/local",
            "cluster_name": cluster_config_dict["cluster_name"],
            "ssh_hostname": (
                cluster_config_dict["ssh_hostname"] if "ssh_hostname" in cluster_config_dict else cluster_config_dict["domain"]
            ),
            "maia_dashboard": {"enabled": True, "token": os.environ.get("rancher_token", "")},
        }
    ]

    if os.environ.get("MAIA_DASHBOARD_OIDC_AUTHENTICATION", False):

        port = get_api_port(os.environ["K8S_DISTRIBUTION"])
        maia_dashboard_values["clusters"][0]["api"] = f"https://{cluster_config_dict['domain']}:{port}"
        maia_dashboard_values["clusters"][0]["maia_dashboard"]["token"] = ""

    maia_dashboard_values["env"].extend(
        [
            {"name": "ARGOCD_CLUSTER", "value": cluster_config_dict["cluster_name"]},
        ]
    )

    # Access Project Pages

    maia_dashboard_values["env"].extend(
        [
            {"name": "CLUSTER_CONFIG_PATH", "value": "/mnt/dashboard-config"},
        ]
    )

    # Access KubeFlow and XNAT
    maia_dashboard_values["env"].extend(
        [
            {"name": "GLOBAL_NAMESPACES", "value": "xnat,istio-system"},
        ]
    )

    # Self Signed Certificate
    if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
        maia_dashboard_values["env"].extend(
            [
                {"name": "OIDC_CA_BUNDLE", "value": "/mnt/dashboard-config/ca.crt"},
            ]
        )

        maia_dashboard_values["ca_crt"] = open(Path(cluster_config_dict["rootCA"])).read()

        maia_dashboard_values["clusters"][0]["selfsigned"] = True
    elif "staging_certificates" in cluster_config_dict and cluster_config_dict["staging_certificates"]:
        maia_dashboard_values["env"].extend(
            [
                {"name": "OIDC_CA_BUNDLE", "value": "/mnt/dashboard-config/ca.crt"},
            ]
        )
        maia_dashboard_values["ca_crt"] = open(Path(cluster_config_dict["externalCA"]["cert"])).read()
        maia_dashboard_values["clusters"][0]["selfsigned"] = True
        if "traefik_resolver" in cluster_config_dict:
            maia_dashboard_values["clusters"][0]["traefik_resolver"] = cluster_config_dict["traefik_resolver"]
        elif "nginx_cluster_issuer" in cluster_config_dict:
            maia_dashboard_values["clusters"][0]["nginx_cluster_issuer"] = cluster_config_dict["nginx_cluster_issuer"]
    else:
        if "traefik_resolver" in cluster_config_dict:
            maia_dashboard_values["clusters"][0]["traefik_resolver"] = cluster_config_dict["traefik_resolver"]
        elif "nginx_cluster_issuer" in cluster_config_dict:
            maia_dashboard_values["clusters"][0]["nginx_cluster_issuer"] = cluster_config_dict["nginx_cluster_issuer"]

    # Deploy with ArgoCD

    maia_dashboard_values["clusters"][0].update(
        {
            "ssh_port_type": cluster_config_dict["ssh_port_type"],
            "port_range": cluster_config_dict["port_range"],
            "shared_storage_class": cluster_config_dict["shared_storage_class"],
            "storage_class": cluster_config_dict["storage_class"],
            "domain": cluster_config_dict["domain"],
            "url_type": cluster_config_dict["url_type"],
            "argocd_destination_cluster_address": (
                cluster_config_dict["argocd_destination_cluster_address"]
                if "argocd_destination_cluster_address" in cluster_config_dict
                else "https://kubernetes.default.svc"
            ),
            "services": {
                "argocd": "https://argocd." + cluster_config_dict["domain"],
                "dashboard": "https://dashboard." + cluster_config_dict["domain"],
                "traefik": "https://traefik." + cluster_config_dict["domain"],
                "grafana": "https://grafana." + cluster_config_dict["domain"],
                "keycloak": "https://iam." + cluster_config_dict["domain"] + "/admin/maia/console/",
                "login": "https://login." + cluster_config_dict["domain"],
                "rancher": "https://mgmt." + cluster_config_dict["domain"],
                "registry": "https://registry." + cluster_config_dict["domain"],
                "minio": "https://minio." + cluster_config_dict["domain"],
            },
        }
    )
    if "metallb_ip_pool" in cluster_config_dict:
        maia_dashboard_values["clusters"][0]["metallb_ip_pool"] = cluster_config_dict["metallb_ip_pool"]
    if "maia_metallb_ip" in cluster_config_dict:
        maia_dashboard_values["clusters"][0]["maia_metallb_ip"] = cluster_config_dict["maia_metallb_ip"]
    if "metallb_shared_ip" in cluster_config_dict:
        maia_dashboard_values["clusters"][0]["metallb_shared_ip"] = cluster_config_dict["metallb_shared_ip"]

    maia_dashboard_values["env"].extend(
        [
            {"name": "maia_project_chart", "value": os.environ.get("maia_project_chart", "maia-project")},
            {"name": "maia_project_repo", "value": os.environ.get("maia_project_repo", "https://minnelab.github.io/MAIA/")},
            {"name": "maia_project_version", "value": os.environ.get("maia_project_version", maia_project_chart_version)},
            {"name": "ADMIN_GROUP", "value": cluster_config_dict.get("admin_group", "admin")},
            {"name": "admin_group_ID", "value": os.environ.get("admin_group_ID", "MAIA:admin")},
            {"name": "USERS_GROUP", "value": cluster_config_dict.get("users_group", "users")},
            {"name": "argocd_namespace", "value": "argocd"},
            {"name": "ARGOCD_SERVER", "value": "https://argocd." + cluster_config_dict["domain"]},
            {"name": "ARGOCD_PASSWORD", "value": os.environ["ARGOCD_PASSWORD"]},
        ]
    )

    # Dev Branch
    if os.environ.get("DEV_BRANCH") is not None:
        maia_dashboard_values["env"].extend(
            [
                {"name": "DEV_BRANCH", "value": os.environ["DEV_BRANCH"]},
            ]
        )
    if os.environ.get("GIT_EMAIL") is not None:
        maia_dashboard_values["env"].extend(
            [
                {"name": "GIT_EMAIL", "value": os.environ["GIT_EMAIL"]},
            ]
        )
    if os.environ.get("GIT_NAME") is not None:
        maia_dashboard_values["env"].extend(
            [
                {"name": "GIT_NAME", "value": os.environ["GIT_NAME"]},
            ]
        )
    if os.environ.get("GPG_KEY") is not None:
        maia_dashboard_values["env"].extend(
            [
                {"name": "GPG_KEY", "value": "/var/keys/gpg.key"},
            ]
        )
        with open(os.environ["GPG_KEY"], "r") as f:
            maia_dashboard_values["gpg_key"] = f.read()

    if (
        os.environ.get("DEV_BRANCH") is not None
        or os.environ.get("GIT_EMAIL") is not None
        or os.environ.get("GIT_NAME") is not None
        or os.environ.get("GPG_KEY") is not None
    ):
        maia_dashboard_values["image"]["tag"] = maia_dashboard_image_version + maia_dashboard_dev_tag_suffix
        maia_dashboard_values["image"]["repository"] = f"{default_registry}/maia-dashboard-dev"

    ### MinIO Configuration
    maia_dashboard_values["env"].extend(
        [
            {"name": "MINIO_URL", "value": "minio:80"},
            {"name": "MINIO_ACCESS_KEY", "value": "maia-admin"},
            {"name": "MINIO_SECRET_KEY", "value": os.environ["minio_admin_password"]},
            {"name": "MINIO_SECURE", "value": "False"},
            {"name": "BUCKET_NAME", "value": "maia-envs"},
            {"name": "MINIO_CONSOLE_URL", "value": f"https://minio.{cluster_config_dict['domain']}/browser/maia-envs"},
        ]
    )

    ## MONAI Toolkit Image and Orthanc
    maia_dashboard_values["env"].extend(
        [
            {"name": "MONAI_TOOLKIT_IMAGE", "value": f"{default_registry}/monai-toolkit"},
        ]
    )

    ## MAIA Registry where  the MAIA images can be pulled from, can also be maiacloudai, default is ghcr.io/minnelab
    maia_dashboard_values["env"].extend(
        [
            {"name": "MAIA_REGISTRY", "value": os.environ.get("MAIA_REGISTRY", "ghcr.io/minnelab")},
        ]
    )

    ## MySQL Configuration

    if "mysql_dashboard_password" in os.environ:
        db_password = os.environ["mysql_dashboard_password"]
    else:
        db_password = generate_human_memorable_password()

    maia_dashboard_values["env"].extend(
        [
            {"name": "DB_ENGINE", "value": "mysql"},
            {"name": "DB_NAME", "value": "mysql"},
            {"name": "DB_HOST", "value": "maia-admin-maia-dashboard-mysql"},
            {"name": "DB_PORT", "value": "3306"},
            {"name": "DB_USERNAME", "value": "maia-admin"},
            {"name": "DB_PASS", "value": db_password},
        ]
    )
    maia_dashboard_values["mysql"].update(
        {
            "mysqlRootPassword": db_password,
            "mysqlUser": "maia-admin",
            "mysqlPassword": db_password,
            "mysqlDatabase": "mysql",
        }
    )

    # Email Notification Systen

    if (
        "SMTP_SENDER_EMAIL" in os.environ
        and "SMTP_SERVER" in os.environ
        and "SMTP_PORT" in os.environ
        and "SMTP_PASSWORD" in os.environ
    ):
        maia_dashboard_values["env"].extend(
            [
                {"name": "SMTP_SENDER_EMAIL", "value": os.environ["SMTP_SENDER_EMAIL"]},
                {"name": "SMTP_SERVER", "value": os.environ["SMTP_SERVER"]},
                {"name": "SMTP_PORT", "value": os.environ["SMTP_PORT"]},
                {"name": "SMTP_PASSWORD", "value": os.environ["SMTP_PASSWORD"]},
            ]
        )

    # Webhook and Support URL
    if "WEBHOOK_URL" in os.environ and "SUPPORT_URL" in os.environ:
        maia_dashboard_values["env"].extend(
            [
                {"name": "WEBHOOK_URL", "value": os.environ["WEBHOOK_URL"]},
                {"name": "SUPPORT_URL", "value": os.environ["SUPPORT_URL"]},
            ]
        )

    # MAIA-Chatbot Configuration
    if "OPENWEBAI_API_KEY" in os.environ and "OPENWEBAI_URL" in os.environ and "OPENWEBAI_MODEL" in os.environ:
        maia_dashboard_values["env"].extend(
            [
                {"name": "OPENWEBAI_API_KEY", "value": os.environ["OPENWEBAI_API_KEY"]},
                {"name": "OPENWEBAI_URL", "value": os.environ["OPENWEBAI_URL"]},
                {"name": "OPENWEBAI_MODEL", "value": os.environ["OPENWEBAI_MODEL"]},
            ]
        )

    # GPU Configuration
    maia_dashboard_values["gpuList"] = os.environ.get("gpu_list", [])

    # MAIA Projects Configuration
    if "maia_projects" in os.environ:
        projects = os.environ["maia_projects"].split(",")
        maia_dashboard_values["maia_projects"] = []
        for project in projects:
            with open(project, "r") as f:
                project_dict = json.load(f)
            maia_dashboard_values["maia_projects"].append(project_dict)

    # CIFS
    # MAIA Segmentation Portal
    # GPU Booking
    Path(config_folder).joinpath(project_id, "maia_dashboard_values").mkdir(parents=True, exist_ok=True)
    with open(Path(config_folder).joinpath(project_id, "maia_dashboard_values", "maia_dashboard_values.yaml"), "w") as f:
        f.write(OmegaConf.to_yaml(maia_dashboard_values))

    return {
        "namespace": maia_dashboard_values["namespace"],
        "release": f"{project_id}-dashboard",
        "chart": (
            maia_dashboard_values["chart_name"] if maia_dashboard_chart_type == "helm_repo" else maia_dashboard_values["path"]
        ),
        "repo": maia_dashboard_values["repo_url"],
        "version": maia_dashboard_values["chart_version"],
        "values": str(Path(config_folder).joinpath(project_id, "maia_dashboard_values", "maia_dashboard_values.yaml")),
    }


def create_rancher_values(config_folder, project_id, cluster_config_dict):
    """
    Generates Rancher values configuration and writes it to a YAML file.

    Parameters
    ----------
    config_folder : str
        The path to the configuration folder.
    project_id : str
        The project identifier.
    cluster_config_dict : dict
        A dictionary containing cluster configuration details.

    Returns
    -------
    dict
        A dictionary containing Rancher deployment details including namespace, repo URL,
        chart version, values file path, release name, and chart name.
    """

    rancher_values = {
        "namespace": "cattle-system",
        "repo_url": "https://releases.rancher.com/server-charts/latest",
        "chart_name": "rancher",
        "chart_version": rancher_chart_version,
    }  # TODO: Change this to updated values

    rancher_values.update(
        {
            "hostname": "mgmt." + cluster_config_dict["domain"],
            "ingress": {"extraAnnotations": {}, "tls": {"source": "letsEncrypt"}},
            "letsEncrypt": {
                "email": cluster_config_dict["ingress_resolver_email"],
                "ingress": {"class": cluster_config_dict["ingress_class"]},
            },
            "bootstrapPassword": os.environ.get("rancher_password", ""),
        }
    )

    if cluster_config_dict["ingress_class"] == "maia-core-traefik":
        rancher_values["ingress"]["extraAnnotations"]["traefik.ingress.kubernetes.io/router.entrypoints"] = "websecure"
        rancher_values["ingress"]["extraAnnotations"]["traefik.ingress.kubernetes.io/router.tls"] = "true"
        rancher_values["ingress"]["tls"]["secretName"] = None

        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            ...
        else:
            rancher_values["ingress"]["extraAnnotations"]["traefik.ingress.kubernetes.io/router.tls.certresolver"] = (
                cluster_config_dict["traefik_resolver"]
            )
    elif cluster_config_dict["ingress_class"] == "nginx":
        if "selfsigned" in cluster_config_dict and cluster_config_dict["selfsigned"]:
            ...
        else:
            rancher_values["ingress"]["extraAnnotations"]["cert-manager.io/cluster-issuer"] = "cluster-issuer"

    Path(config_folder).joinpath(project_id, "rancher_values").mkdir(parents=True, exist_ok=True)

    with open(Path(config_folder).joinpath(project_id, "rancher_values", "rancher_values.yaml"), "w") as f:
        f.write(OmegaConf.to_yaml(rancher_values))

    return {
        "namespace": rancher_values["namespace"],
        "repo": rancher_values["repo_url"],
        "version": rancher_values["chart_version"],
        "values": str(Path(config_folder).joinpath(project_id, "rancher_values", "rancher_values.yaml")),
        "release": f"{project_id}-rancher",
        "chart": rancher_values["chart_name"],
    }



import os

dev_distros = ["microk8s", "k0s", "k3s"]


def get_api_port(k8s_distribution):
    port = None
    if k8s_distribution == "microk8s":
        port = 16443
    elif k8s_distribution == "k0s":
        port = 6443
    elif k8s_distribution == "k3s":
        port = 6443
    else:
        raise ValueError(f"K8S_DISTRIBUTION {os.environ['K8S_DISTRIBUTION']} not supported")
    return port


def get_gpu_operator_toolkit(k8s_distribution):

    if k8s_distribution == "microk8s":
        gpu_operator_values_toolkit = {
            "env": [
                {"name": "CONTAINERD_CONFIG", "value": "/var/snap/microk8s/current/args/containerd-template.toml"},
                {"name": "CONTAINERD_SOCKET", "value": "/var/snap/microk8s/common/run/containerd.sock"},
                {"name": "CONTAINERD_RUNTIME_CLASS", "value": "nvidia"},
                {"name": "CONTAINERD_SET_AS_DEFAULT", "value": "true"},
            ]
        }

    elif k8s_distribution == "rke2":
        gpu_operator_values_toolkit = {
            "driver": {"enabled": False},
            "env": [
                {"name": "CONTAINERD_SOCKET", "value": "/run/k3s/containerd/containerd.sock"},
                {"name": "CONTAINERD_CONFIG", "value": "/var/lib/rancher/rke2/agent/etc/containerd/config.toml.tmpl"},
                {"name": "CONTAINERD_RUNTIME_CLASS", "value": "nvidia"},
                {"name": "CONTAINERD_SET_AS_DEFAULT", "value": "true"},
            ],
        }
    elif k8s_distribution == "k0s":
        gpu_operator_values_toolkit = {
            "operator": {"defaultRuntime": "containerd"},
            "toolkit": {
                "env": [
                    {"name": "CONTAINERD_CONFIG", "value": "/etc/k0s/containerd.d/nvidia.toml"},
                    {"name": "CONTAINERD_SOCKET", "value": "/run/k0s/containerd.sock"},
                    {"name": "CONTAINERD_RUNTIME_CLASS", "value": "nvidia"},
                ]
            },
        }
    elif k8s_distribution == "k3s":
        ...
        # TODO: Implement k3s GPU operator toolkit
    else:
        raise ValueError(f"K8S_DISTRIBUTION {k8s_distribution} not supported")
    return gpu_operator_values_toolkit


def get_storage_class(k8s_distribution):
    if k8s_distribution in dev_distros:
        if k8s_distribution == "microk8s":
            storage_class = "microk8s-hostpath"
        elif k8s_distribution == "k0s":
            storage_class = "local-path"
        elif k8s_distribution == "k3s":
            storage_class = "local-path"
    else:
        storage_class = "local-path"
    return storage_class


def get_ingress_class(k8s_distribution):
    if k8s_distribution in dev_distros:
        ingress_class = "maia-core-traefik"
    else:
        ingress_class = "nginx"
    return ingress_class



#! /usr/bin/env python3
from keycloak import KeycloakAdmin
from keycloak import KeycloakOpenIDConnection
import argparse
from argparse import RawTextHelpFormatter
from textwrap import dedent
from pathlib import Path

from loguru import logger

EPILOG = dedent("""
    Example call:
    ::
        {filename} --client_secret <client_secret> --server_url <server_url>
    """.format(filename=Path(__file__).stem))  # noqa: E501


def create_admin_user_and_group(
    server_url: str,
    client_secret: str,
    admin_email="admin@maia.se",
    group_id="users",
    admin_password="admin",
    is_admin=True,
    admin_group_id="admin",
):
    keycloak_connection = KeycloakOpenIDConnection(
        server_url=server_url,
        username="user",
        password="",
        realm_name="maia",
        client_id="maia",
        client_secret_key=client_secret,
        verify=False,
    )

    keycloak_admin = KeycloakAdmin(connection=keycloak_connection)

    try:
        keycloak_admin.create_user(
            {
                "username": admin_email,
                "email": admin_email,
                "emailVerified": True,
                "enabled": True,
                "firstName": "Admin",
                "lastName": "Maia",
                "requiredActions": ["UPDATE_PASSWORD"],
                "credentials": [{"type": "password", "temporary": True, "value": admin_password}],
            }
        )
    except Exception as e:
        logger.error(f"Error creating admin user: {e}")
        pass

    payload = {
        "name": f"MAIA:{group_id}",
        "path": f"/MAIA:{group_id}",
        "attributes": {},
        "realmRoles": [],
        "clientRoles": {},
        "subGroups": [],
        "access": {"view": True, "manage": True, "manageMembership": True},
    }
    try:
        keycloak_admin.create_group(payload)
    except Exception as e:
        logger.error(f"Error creating group: {e}")
        pass

    groups = keycloak_admin.get_groups()

    users = keycloak_admin.get_users()
    for user in users:
        if "email" in user and user["email"] in [admin_email]:
            uid = user["id"]
            for group in groups:
                if group["name"] == f"MAIA:{group_id}":
                    gid = group["id"]
                    keycloak_admin.group_user_add(uid, gid)
                if is_admin:
                    if group["name"] == f"MAIA:{admin_group_id}":
                        gid = group["id"]
                        keycloak_admin.group_user_add(uid, gid)

    client_uuid = keycloak_admin.get_client_id("realm-management")
    if client_uuid is None:
        raise ValueError("Client UUID not found in Keycloak")
    client_roles = keycloak_admin.get_client_roles(client_uuid)
    for group in groups:
        if group["name"] == f"MAIA:{admin_group_id}":
            gid = group["id"]
            keycloak_admin.assign_group_client_roles(
                group_id=gid,
                client_id=client_uuid,
                roles=client_roles,
            )


def create_public_client(server_url: str, client_secret: str):
    keycloak_connection = KeycloakOpenIDConnection(
        server_url=server_url,
        username="user",
        password="",
        realm_name="maia",
        client_id="maia",
        client_secret_key=client_secret,
        verify=False,
    )
    keycloak_admin = KeycloakAdmin(connection=keycloak_connection)
    keycloak_admin.create_client(
        {
            "name": "maia-public",
            "clientId": "maia-public",
            "redirectUris": ["http://localhost:8080"],
            "publicClient": True,
            "standardFlowEnabled": True,
            "directAccessGrantsEnabled": True,
        }
    )


def get_arg_parser():
    parser = argparse.ArgumentParser(
        description="Configure Keycloak",
        epilog=EPILOG,
        formatter_class=RawTextHelpFormatter,
    )
    parser.add_argument("--client_secret", type=str, required=True, help="The client secret to use")
    parser.add_argument("--server_url", type=str, required=True, help="The server URL to configure")
    parser.add_argument("--admin_email", type=str, required=True, help="The admin email to use")
    parser.add_argument("--admin_password", type=str, required=False, default="admin", help="The admin password to use")
    parser.add_argument("--admin_group_id", type=str, required=False, default="admin", help="The admin group ID to use")
    return parser


def main():
    args = get_arg_parser().parse_args()
    create_public_client(
        server_url=args.server_url,
        client_secret=args.client_secret,
    )
    create_admin_user_and_group(
        server_url=args.server_url,
        client_secret=args.client_secret,
        admin_email=args.admin_email,
        admin_password=args.admin_password,
        admin_group_id=args.admin_group_id,
    )


if __name__ == "__main__":
    main()



from __future__ import annotations

import smtplib
import ssl
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from loguru import logger


def send_email_approved_project_registration(
    project_name, project_owner, support_link, dashboard_url, smtp_sender_email, smtp_server, smtp_port, smtp_password
):

    message = MIMEMultipart()
    message["Subject"] = f"{project_name} Project Approved"
    message["From"] = f"MAIA Admin Team <{smtp_sender_email}>"
    message["To"] = project_owner

    html = """\
    <html>
        <head></head>
        <body>
            <p>Welcome to MAIA!</p>
            <p>Your project <b>{}</b> has been approved and you can now access the the project workspace at:<br>
            <a href="{}namespaces/{}">{}namespaces/{}</a></p>
            
            <p>In the project workspace page you can find all the links and applications available for your project.</p>
            <br>
            <p>If you have any questions or need further assistance, feel free to join our Discord community:<br>
            <a href="{}">{}</a></p>
            </p>

            <p>Best regards,</p>
            <p>The MAIA Admin Team</p>
        </body>
    </html>
    """.format(project_name, dashboard_url, project_name, dashboard_url, project_name, support_link, support_link)

    part1 = MIMEText(html, "html")
    message.attach(part1)

    _ = ssl.create_default_context()

    try:

        if not smtp_server or not smtp_sender_email or not smtp_password:
            raise ValueError("Missing required email environment variables.")
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.ehlo()  # identify ourselves to SMTP server
            server.starttls()  # encrypt the session
            server.login(smtp_sender_email, smtp_password)
            server.sendmail(smtp_sender_email, project_owner, message.as_string())
        logger.success(f"Project {project_name} registration email sent to {project_owner}")
    except Exception as smtp_error:
        logger.error(f"SMTP error: {smtp_error}")
        raise


def send_email_user_registration_to_group(
    project_name, user_email, support_link, dashboard_url, smtp_sender_email, smtp_server, smtp_port, smtp_password
):

    message = MIMEMultipart()
    message["Subject"] = f"{project_name} Registration Approved"
    message["From"] = f"MAIA Admin Team <{smtp_sender_email}>"
    message["To"] = user_email

    html = """\
    <html>
        <head></head>
        <body>
            <p>Welcome to MAIA!</p>
            <p>Your request to join the group <b>{}</b> has been approved and you can now access the the project workspace at:<br>
            <a href="{}namespaces/{}">{}namespaces/{}</a></p>
            
            <p>In the project workspace page you can find all the links and applications available for your project.</p>
            <br>
            <p>If you have any questions or need further assistance, feel free to join our Discord community:<br>
            <a href="{}">{}</a></p>
            </p>

            <p>Best regards,</p>
            <p>The MAIA Admin Team</p>
        </body>
    </html>
    """.format(project_name, dashboard_url, project_name, dashboard_url, project_name, support_link, support_link)

    part1 = MIMEText(html, "html")
    message.attach(part1)

    _ = ssl.create_default_context()

    try:

        if not smtp_server or not smtp_sender_email or not smtp_password:
            raise ValueError("Missing required email environment variables.")
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.ehlo()  # identify ourselves to SMTP server
            server.starttls()  # encrypt the session
            server.login(smtp_sender_email, smtp_password)
            server.sendmail(smtp_sender_email, user_email, message.as_string())
        logger.success(f"Project {project_name} registration email sent to {user_email}")
    except Exception as smtp_error:
        logger.error(f"SMTP error: {smtp_error}")
        raise


def confirm_request_registration_to_project(
    project_name, user_email, support_link, dashboard_url, smtp_sender_email, smtp_server, smtp_port, smtp_password
):
    message = MIMEMultipart()
    message["Subject"] = f"Confirmation of your request to join the MAIA project {project_name}"
    message["From"] = f"MAIA Admin Team <{smtp_sender_email}>"
    message["To"] = user_email

    html = """\
    <html>
        <head></head>
        <body>
            <p>Welcome to MAIA!</p>
            <p>Your request to join the group <b>{}</b> in now being processed. You will receive a confirmation email once your request is approved.</p>
            <br>
            <p>If you have any questions or need further assistance, feel free to join our Discord community:<br>
            <a href="{}">{}</a></p>
            </p>
            <p>Best regards,</p>
            <p>The MAIA Admin Team</p>
        </body>
    </html>
    """.format(project_name, support_link, support_link)

    part1 = MIMEText(html, "html")
    message.attach(part1)

    _ = ssl.create_default_context()

    try:
        if not smtp_server or not smtp_sender_email or not smtp_password:
            raise ValueError("Missing required email environment variables.")
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.ehlo()  # identify ourselves to SMTP server
            server.starttls()  # encrypt the session
            server.login(smtp_sender_email, smtp_password)
            server.sendmail(smtp_sender_email, user_email, message.as_string())
            logger.success(f"Confirmation email sent to {user_email} for project {project_name}")
        return True
    except Exception as smtp_error:
        logger.error(f"SMTP error: {smtp_error}")
        return False


def confirm_request_registration_for_group(
    group_name, user_email, support_link, dashboard_url, smtp_sender_email, smtp_server, smtp_port, smtp_password
):
    message = MIMEMultipart()
    message["Subject"] = f"Confirmation of your request to register a new MAIA project {group_name}"
    message["From"] = f"MAIA Admin Team <{smtp_sender_email}>"
    message["To"] = user_email

    html = """\
    <html>
        <head></head>
        <body>
            <p>Welcome to MAIA!</p>
            <p>Your request to register a new project <b>{}</b> is now being processed. You will receive a confirmation email once your request is approved.</p>
            <br>
            <p>If you have any questions or need further assistance, feel free to join our Discord community:<br>
            <a href="{}">{}</a></p>
            </p>
            <p>Best regards,</p>
            <p>The MAIA Admin Team</p>
        </body>
    </html>
    """.format(group_name, support_link, support_link)

    part1 = MIMEText(html, "html")
    message.attach(part1)

    _ = ssl.create_default_context()

    try:
        if not smtp_server or not smtp_sender_email or not smtp_password:
            raise ValueError("Missing required email environment variables.")
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.ehlo()  # identify ourselves to SMTP server
            server.starttls()  # encrypt the session
            server.login(smtp_sender_email, smtp_password)
            server.sendmail(smtp_sender_email, user_email, message.as_string())
            logger.success(f"Confirmation email sent to {user_email} for group {group_name}")
        return True
    except Exception as smtp_error:
        logger.error(f"SMTP error: {smtp_error}")
        return False


def send_email_approved_registration_email(
    email, temp_password, login_url, smtp_sender_email, smtp_server, smtp_port, smtp_password
):
    message = MIMEMultipart()
    message["Subject"] = "Your MAIA Account has been approved"
    message["From"] = f"MAIA Admin Team <{smtp_sender_email}>"
    message["To"] = email

    html = """\
    <html>
        <head></head>
        <body>
            <p>Welcome to MAIA!</p>
            <p>Your MAIA account has been approved and you can now log in to MAIA at the following link: <a href="{}">{}</a></p>
            <p>Your temporary password is: {}</p>
            <p>Please change your password after logging in.</p>
            <br>
            <p>Best regards,</p>
            <p>The MAIA Admin Team</p>
        </body>
    </html>
    """.format(login_url, login_url, temp_password)

    part1 = MIMEText(html, "html")
    message.attach(part1)

    _ = ssl.create_default_context()

    try:
        if not smtp_server or not smtp_sender_email or not smtp_password:
            raise ValueError("Missing required email environment variables.")
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.ehlo()  # identify ourselves to SMTP server
            server.starttls()  # encrypt the session
            server.login(smtp_sender_email, smtp_password)
            server.sendmail(smtp_sender_email, email, message.as_string())
            logger.success(f"Approved registration email sent to {email}")
        return True
    except Exception as smtp_error:
        logger.error(f"SMTP error: {smtp_error}")
        return False
